package com.dothantech.lpapi.cordova;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Base64;
import android.util.Log;

import com.dothantech.lpapi.IAtBitmap;
import com.dothantech.lpapi.LPAPI;
import com.dothantech.lpapi.LPAPI.Callback;
import com.dothantech.printer.IDzPrinter;
import com.dothantech.printer.IDzPrinter.PrintParamName;
import com.dothantech.printer.IDzPrinter.PrintProgress;
import com.dothantech.printer.IDzPrinter.PrinterAddress;
import com.dothantech.printer.IDzPrinter.PrinterInfo;
import com.dothantech.printer.IDzPrinter.PrinterState;
import com.dothantech.printer.IDzPrinter.ProgressInfo;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

public class LPAPIBridge {

    private static class JobInfo {
        public double width;
        public double height;
        public int orientation;
        public String jobName;
    }

    public static final String MODELS = "models";
    public static final String CLIENT_TYPE = "clientType";
    private static final String TAG = "LPAPIBridge";
    private static final String PARAMS_NAME = "name";
    private static final String PARAMS_VALUE = "value";
    private static final String PARAMS_X = "x";
    private static final String PARAMS_X1 = "x1";
    private static final String PARAMS_X2 = "x2";
    private static final String PARAMS_Y = "y";
    private static final String PARAMS_Y1 = "y1";
    private static final String PARAMS_Y2 = "y2";
    private static final String PARAMS_WIDTH = "width";
    private static final String PARAMS_HEIGHT = "height";
    private static final String PARAMS_ORIENTATION = "orientation";
    private static final String PARAMS_TEXT = "text";
    private static final String PARAMS_FONT_HEIGHT = "fontHeight";
    private static final String PARAMS_FONT_STYLE = "fontStyle";
    private static final String PARAMS_FONT_NAME = "fontName";
    private static final String PARAMS_TEXT_HEIGHT = "textHeight";
    private static final String PARAMS_LINE_WIDTH = "lineWidth";
    private static final String PARAMS_CORNER = "corner";
    private static final String PARAMS_CORNER_WIDTH = "cornerWidth";
    private static final String PARAMS_CORNER_HEIGHT = "cornerHeight";
    private static final String PARAMS_RADIUS = "radius";
    private static final String PARAMS_DASH_LEN = "dashLen";
    private static final String PARAMS_IMAGE = "image";
    private static final String PARAMS_THRESHOLD = "threshold";
    private static final String PARAMS_TYPE = "type";
    private static final String PARAMS_ALIGNMENT = "alignment";
    private static final String PARAMS_LINESPACE = "linespace";
    private static final String PARAMS_COLOR = "color";
    private static final String PARAMS_ECC_LEVEL = "eccLevel";

    private static final String PARAMS_SHOWN_NAME = "name";
    private static final String PARAMS_MAC_ADDRESS = "macAddress";
    private static final String PARAMS_ADDRESS_TYPE = "addressType";

//    public static final String ERROR_PARAM_ERROR = "参数错误";

    // 相关默认参数
    private static final double CORNER_DEFAULT = 1.5;
    private static final double LINEWIDTH_DEFAULT = 0.3;
    private static final int THRESHOLD_DEFAULT = 192;

//    private static final String REG_IMG_BASE64 = "^data:image/[a-z]{3,4};base64,.*";

    private final LPAPI mApi;
    private String mModels = null;
    private JobInfo mJobInfo;
    private IDiscoveryResponse mDiscoveryCallback;
    private final Map<String, PrinterAddress> mRecentDeviceMap = new HashMap<>();
    private Timer mScanUpdateTimer;
    private Timer mScanTimeoutTimer;
    private IResponse mOpenPrinterCallback;
    private IResponse mPrintCallback;

    public interface IResponse {
        void onResult(boolean success, JSONObject msg);
    }

    public interface IDiscoveryResponse {
        void onStatusChanged(EDiscoveryStatus status, Collection<PrinterAddress> devices);
    }

    public enum EDiscoveryStatus {
        OK,
        DEVICE_FOUND,
        COMPLETE,
        CANCEL,
        STOP,
        FAILED_PERMISSION,
    }

    private static LPAPIBridge sInstance;

    /**
     * LPAPI参数转换实例，不管是同步还是一部，共用一个实例；
     */
    public static LPAPIBridge getInstance() {
        if (sInstance == null)
            sInstance = new LPAPIBridge();

        return sInstance;
    }

    private static boolean isPreviewJob(String jobName) {
        if (TextUtils.isEmpty(jobName)) {
            return false;
        }
        jobName = jobName.toLowerCase();
        return jobName.startsWith("#!#prev") || jobName.startsWith("#!#trans");
    }

    public JSONObject getJsonData(String key, Object value, JSONObject data) {
        if (value != null) {
            if (TextUtils.isEmpty(key)) {
                key = "data";
            }
            if (data == null) {
                data = new JSONObject();
            }
            try {
                data.put(key, value);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            return data;
        }
        //
        return data;
    }

    public JSONObject getJsonData(String key, Object value) {
        return this.getJsonData(key, value, null);
    }

    public static JSONObject getDeviceObject(PrinterAddress device) {
        JSONObject result = new JSONObject();
        if (device == null) return result;
        //
        try {
            result.put(PARAMS_SHOWN_NAME, device.shownName);
            result.put(PARAMS_ADDRESS_TYPE, device.addressType);
            result.put(PARAMS_MAC_ADDRESS, device.macAddress);
            result.put("deviceId", device.macAddress);
            result.put("mac", device.macAddress);
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
        return result;
    }

    private JSONObject getCommitResult(JobInfo info, JSONObject device, JSONArray pages) {
        JSONObject result = new JSONObject();
        if (info == null) {
            return result;
        }
        try {
            result.put("width", info.width);
            result.put("height", info.height);
            result.put("orientation", info.orientation);
            result.put("jobName", info.jobName);
            if (device != null) {
                result.put("device", device);
            }
            if (pages != null) {
                result.put("pages", pages);

            }
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
        return result;
    }

    public static JSONArray getDeviceArray(Collection<PrinterAddress> devices) {
        JSONArray arr = new JSONArray();
        for (PrinterAddress item : devices) {
            if (item != null) {
                JSONObject obj = getDeviceObject(item);
                arr.put(obj);
            }
        }
        return arr;
    }

    public LPAPIBridge() {
        this.mApi = LPAPI.Factory.createInstance(new Callback() {

            @Override
            public void onStateChange(PrinterAddress address, PrinterState state) {
//                Log.i(TAG, String.format("OnPrinterStateChanged: {stateName: %s}", state.name()));
//                Log.i(TAG, String.format("mOpenPrinterCallback %s null", mOpenPrinterCallback == null ? "==" : "!="));
                if (mOpenPrinterCallback == null) {
                    return;
                }

                if (state == PrinterState.Connected || state == PrinterState.Connected2) {
                    mOpenPrinterCallback.onResult(true, getPrinterInfo());
                } else if (state == PrinterState.Disconnected) {
                    mOpenPrinterCallback.onResult(false, null);
                }
            }

            @Override
            public void onProgressInfo(ProgressInfo info, Object addiInfo) {
//                Log.i(TAG, String.format("OnProgressInfo: %s", info.toString()));
            }

            @Override
            public void onPrinterDiscovery(PrinterAddress address, Object info) {
                if (info != IDzPrinter.GeneralProgress.Info) {
                    Log.i(TAG, "★★★★★★ onPrinterDiscovery, info = " + info);
                }
                if (mDiscoveryCallback != null) {
                    if (info == IDzPrinter.GeneralProgress.Start) {
                        // 启动成功
                        mDiscoveryCallback.onStatusChanged(EDiscoveryStatus.OK, null);
                    } else if (info == IDzPrinter.GeneralProgress.Success) {
                        // 搜索结束
//                        mDiscoveryCallback.onStatusChanged(EDiscoveryStatus.COMPLETE);
                        onDiscoveryFinished(EDiscoveryStatus.COMPLETE);
                    } else if (info == IDzPrinter.GeneralProgress.Info) {
                        // 检测到设备
                        if (!TextUtils.isEmpty(mModels)) {
                            if (isPrinterSupported(address.shownName, mModels)) {
                                if (mScanUpdateTimer != null) {
                                    mRecentDeviceMap.put(address.macAddress, address);
                                } else {
                                    List<PrinterAddress> list = new ArrayList<>();
                                    list.add(address);
                                    mDiscoveryCallback.onStatusChanged(EDiscoveryStatus.DEVICE_FOUND, list);
                                }
                            } else {
                                Log.i(TAG, String.format("---- 受限的设备[%s]: %s => %s", mModels, address.shownName, address.macAddress));
                            }
                        } else {
                            if (mScanUpdateTimer != null) {
                                mRecentDeviceMap.put(address.macAddress, address);
                            } else {
                                List<PrinterAddress> list = new ArrayList<>();
                                list.add(address);
                                mDiscoveryCallback.onStatusChanged(EDiscoveryStatus.DEVICE_FOUND, list);
                            }
                        }
                    } else if (info == IDzPrinter.GeneralProgress.Cancelled) {
                        // 搜索被取消
                        onDiscoveryFinished(EDiscoveryStatus.CANCEL);
                        stopScanTimer();
                    }
                }
            }

            @Override
            public void onPrintProgress(PrinterAddress address, IDzPrinter.PrintData printData, PrintProgress progress, Object addiInfo) {
                if (mPrintCallback == null)
                    return;

                if (progress == PrintProgress.Success) {
                    JSONArray pages = getJobPages();
                    mPrintCallback.onResult(true, getCommitResult(mJobInfo, getDeviceObject(address), pages));
                } else if (progress == PrintProgress.Failed) {
                    mPrintCallback.onResult(false, null);
                }
            }
        });
    }

    public LPAPI api() {
        return this.mApi;
    }

    public JSONArray getAllPrinterAddresses(JSONObject params) {
        params = params == null ? new JSONObject() : params;
        String models = params.optString(PARAMS_NAME);
        List<PrinterAddress> addressList = this.mApi.getAllPrinterAddresses(models);
        return addressList != null ? getDeviceArray(addressList) : new JSONArray();
    }

    public void setPrinterModels(String models) {
        if (!TextUtils.isEmpty(models)) {
            this.mModels = models;
        }
    }

    public boolean isPrinterSupported(String deviceName, String models) {
        if (TextUtils.isEmpty(models)) {
            models = this.mModels;
        }
        return mApi.isPrinterSupported(deviceName, models);
    }

    private void onDiscoveryFinished(EDiscoveryStatus status) {
        if (mDiscoveryCallback != null) {
            mDiscoveryCallback.onStatusChanged(status, null);
            mDiscoveryCallback = null;
        }
    }

    private void startScanTimer(int timeout, int interval) {
        // 超时时间到了之后，自动停止蓝牙搜索操作；
        if (timeout > 0) {
            mScanTimeoutTimer = new Timer();
            mScanTimeoutTimer.schedule(new TimerTask() {
                @Override
                public void run() {
                    stopScanTimer();
                    // 停止搜索
                    stopBleDiscovery();
                }
            }, timeout);
        }
        // 定时检查搜索结果，并进行消息回调处理
        if (interval > 0) {
            mScanUpdateTimer = new Timer();
            mScanUpdateTimer.schedule(new TimerTask() {
                @Override
                public void run() {

                    // 检查 recentMap 重是否有设备，有的话，进行回调处理
                    if (!mRecentDeviceMap.isEmpty() && mDiscoveryCallback != null) {
                        mDiscoveryCallback.onStatusChanged(EDiscoveryStatus.DEVICE_FOUND, mRecentDeviceMap.values());
                        // 清空 recentMap
                        mRecentDeviceMap.clear();
                    }
                }
            }, interval, interval);
        }
    }

    private void stopScanTimer() {
        if (mScanTimeoutTimer != null) {
            mScanTimeoutTimer.cancel();
            mScanTimeoutTimer = null;
        }
        if (mScanUpdateTimer != null) {
            mScanUpdateTimer.cancel();
            mScanUpdateTimer = null;
        }
    }

    public void startBleDiscovery(JSONObject options, IDiscoveryResponse callback) {
        String models = options.optString(MODELS, "");
        int timeout = options.optInt("timeout");
        int interval = options.optInt("interval");
        //
        mRecentDeviceMap.clear();
        mDiscoveryCallback = callback;
        setPrinterModels(models);
        //
        startScanTimer(timeout, interval);
        //
        this.mApi.discovery();
    }

    public void stopBleDiscovery() {
        this.mApi.stopDiscovery();
        //
        stopScanTimer();
        //
        onDiscoveryFinished(EDiscoveryStatus.STOP);
    }

    public boolean openPrinter(JSONObject params, IResponse response) {
        // 如果打印机已经连接，并且打印机名称与当前打印机名称一致，则直接返回
        this.mOpenPrinterCallback = response;
        String name = params.optString(PARAMS_NAME, "");
//        String deviceId = params.optString("deviceId", "");
//        if (TextUtils.isEmpty(deviceId))
//            deviceId = params.optString("mac", "");
        int clientType = params.optInt("clientType", -1);
        //
        if (this.isPrinterOpened()) {
            String oldName = this.mApi.getPrinterName();
            if (name.equalsIgnoreCase(oldName)) {
                response.onResult(true, this.getPrinterInfo());
                return true;
            }
        }
        if (clientType >= 0)
            mApi.setCT((short) clientType);
        //
        return mApi.openPrinter(name);
    }

    /**
     * 获取当前打印机名称；
     */
    public String getPrinterName() {
        return this.mApi.getPrinterName();
    }

    /**
     * 获取当前已连接打印机的详细信息；
     */
    public JSONObject getPrinterInfo() {
        JSONObject result = new JSONObject();
        PrinterInfo printer = this.mApi.getPrinterInfo();
        try {
            if (printer != null) {
                // 设备信息
                result.put(PARAMS_SHOWN_NAME, printer.deviceName);
                result.put(PARAMS_MAC_ADDRESS, printer.deviceAddress);
                result.put(PARAMS_ADDRESS_TYPE, printer.deviceAddrType);
                result.put("deviceId", printer.deviceAddress);
                result.put("mac", printer.deviceAddress);
                //
                result.put("deviceVersion", printer.deviceVersion);
                result.put("softwareVersion", printer.softwareVersion);
                result.put("deviceWidth", printer.deviceWidth);
                result.put("deviceType", printer.deviceType);
                result.put("deviceDPI", printer.deviceDPI);
                //
                result.put("manufacturer", printer.manufacturer);
                result.put("seriesName", printer.seriesName);
                result.put("devIntName", printer.devIntName);
                result.put("hardwareFlags", printer.hardwareFlags);
                result.put("softwareFlags", printer.softwareFlags);
                result.put("mcuId", printer.mcuId);
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }

        return result;
    }

    /**
     * 获取当前打印机连接状态；
     */
    public JSONObject getPrinterState() {
        PrinterState state = this.mApi.getPrinterState();
        JSONObject obj = this.getJsonData("name", state.name());
        this.getJsonData("value", state.ordinal(), obj);
        return obj;
    }

    /**
     * 从配对列表中获取指定型号的第一台打印机；
     */
    public JSONObject getFirstPrinter(JSONObject params) {
        String model = (params == null ? "" : params.optString(PARAMS_NAME));
        List<PrinterAddress> list = this.mApi.getAllPrinterAddresses(model);
        return list != null && !list.isEmpty() ? getDeviceObject(list.get(0)) : new JSONObject();
    }

    public boolean isPrinterOpened() {
        return this.mApi.isPrinterOpened();
    }

    public void cancel() {
        this.mApi.cancel();
    }

    public void closePrinter() {
        this.mApi.closePrinter();
    }

    public boolean printImage(JSONObject params, IResponse response) {
        if (params == null)
            return false;

        String image = params.optString(PARAMS_IMAGE);
        if (TextUtils.isEmpty(image))
            image = params.optString("data");
        if (TextUtils.isEmpty(image)) {
            return false;
        }
        //
        Bitmap bitmap = decodeBase64(image);
        if (bitmap == null)
            return false;

        this.mPrintCallback = response;
        return this.mApi.printBitmap(bitmap, this.getPrintParams(params));
    }

    private Bundle getPrintParams(JSONObject params) {
        Bundle printParam = new Bundle();
        if (params == null)
            return printParam;

        int gapType = params.optInt(PrintParamName.GAP_TYPE, -1);
        if (gapType < 0)
            gapType = params.optInt("gapType", -1);
        int printDensity = params.optInt(PrintParamName.PRINT_DENSITY, -1);
        if (printDensity < 0)
            printDensity = params.optInt("printDarkness", -1);
        int printSpeed = params.optInt(PrintParamName.PRINT_SPEED, -1);
        if (printSpeed < 0)
            printSpeed = params.optInt("printSpeed", -1);
        int printCopys = params.optInt(PrintParamName.PRINT_COPIES, 0);
        if (printCopys <= 0)
            printCopys = params.optInt("printCopies", 0);

        if (gapType >= 0)
            printParam.putInt(PrintParamName.GAP_TYPE, gapType);
        if (printDensity >= 0)
            printParam.putInt(PrintParamName.PRINT_DENSITY, printDensity);
        if (printSpeed >= 0)
            printParam.putInt(PrintParamName.PRINT_SPEED, printSpeed);
        if (printCopys > 0)
            printParam.putInt(PrintParamName.PRINT_COPIES, printCopys);

        return printParam;
    }

    public boolean startJob(JSONObject params) {
        if (params == null)
            return false;

        mJobInfo = new JobInfo();
        mJobInfo.width = params.optDouble(PARAMS_WIDTH);
        mJobInfo.height = params.optDouble(PARAMS_HEIGHT);
        mJobInfo.orientation = params.optInt(PARAMS_ORIENTATION);
        mJobInfo.jobName = params.optString("jobName", "");
        int bgColor = params.optInt("backgroundColor", -1);
        if (bgColor < 0)
            bgColor = params.optInt("background", Color.WHITE);
        boolean isPreview = LPAPIBridge.isPreviewJob(mJobInfo.jobName);
        // 设备打印任务的背景色
        mApi.setBackground((isPreview && bgColor >= 0) ? bgColor : Color.WHITE);
        //
        boolean res = this.mApi.startJob(mJobInfo.width, mJobInfo.height, mJobInfo.orientation);
        if (!res) {
            return false;
        }
        //
        if (LPAPIBridge.isPreviewJob(mJobInfo.jobName)) {
            String imageUrl = params.optString("imageUrl");
            String bgImage = params.optString("backgroundImage");
            if (!TextUtils.isEmpty(imageUrl)) {
                mApi.drawImageWithThreshold(imageUrl, 0, 0, mJobInfo.width, mJobInfo.height, 257);
            } else if (!TextUtils.isEmpty(bgImage)) {
                Bitmap bmp = LPAPIBridge.decodeBase64(bgImage);
                if (bmp != null) {
                    mApi.drawBitmapWithThreshold(bmp, 0, 0, mJobInfo.width, mJobInfo.height, 257);
                }
            }
        }
        return true;
    }

    public void abortJob() {
        this.mApi.abortJob();
    }

    public boolean commitJob(JSONObject params, IResponse response) {
        if (this.mJobInfo == null) {
            return false;
        }
        // 如果打印任务名称为预览，则直接调用 endJob 即可，不需要打印
        if (LPAPIBridge.isPreviewJob(this.mJobInfo.jobName)) {
            this.mApi.endJob();
            JSONArray pages = getJobPages();
            response.onResult(true, getCommitResult(mJobInfo, null, pages));
            return true;
        }
        // 执行打印操作
        this.mPrintCallback = response;
        this.mJobInfo = null;
        return this.mApi.commitJobWithParam(getPrintParams(params));
    }

    public boolean startPage() {
        return this.mApi.startPage();
    }

    public void endPage() {
        this.mApi.endPage();
    }

    public void endJob() {
        this.mApi.endJob();
    }

    public JSONArray getJobPages() {
        List<Bitmap> pages = this.mApi.getJobPages();
        if (pages == null || pages.isEmpty())
            return null;
        //
        JSONArray array = new JSONArray();
        for (Bitmap bitmap : pages) {
            array.put(encodeBase64(bitmap));
        }
        return array;
    }

    public void setDrawParam(JSONObject params) {
        if (params == null)
            return;

        String name = params.optString(PARAMS_NAME);
        Object value = params.optString(PARAMS_VALUE);
        if (TextUtils.isEmpty(name))
            return;

        this.mApi.setDrawParam(name, value);
    }

    // -----------------------------参数设置--------------------- //

    public int getItemOrientation() {
        return this.mApi.getItemOrientation();
    }

    public boolean setItemOrientation(JSONObject params) {
        if (params == null)
            return false;

        int orientation = params.optInt(PARAMS_ORIENTATION, -1);
        if (orientation == -1)
            return false;

        this.mApi.setItemOrientation(orientation);
        return true;
    }

    public int getItemHorizontalAlignment() {
        return this.mApi.getItemHorizontalAlignment();
    }

    public boolean setItemHorizontalAlignment(JSONObject params) {
        if (params == null)
            return false;

        int alignment = params.optInt(PARAMS_ALIGNMENT, -1);
        if (alignment == -1)
            return false;

        this.mApi.setItemHorizontalAlignment(alignment);
        return true;
    }

    public int getItemVerticalAlignment() {
        return this.mApi.getItemVerticalAlignment();
    }

    public boolean setItemVerticalAlignment(JSONObject params) {
        if (params == null)
            return false;

        int alignment = params.optInt(PARAMS_ALIGNMENT, -1);
        if (alignment == -1)
            return false;

        this.mApi.setItemVerticalAlignment(alignment);
        return true;
    }

//    public int getItemPenAlignment() {
//        return this.mApi.getItemPenAlignment();
//    }

//    public boolean setItemPenAlignment(JSONObject params) {
//        if (params == null)
//            return false;
//
//        int penAlignment = params.optInt(PARAMS_ALIGNMENT, -1);
//        if (penAlignment == -1)
//            return false;
//
//        this.mApi.setItemPenAlignment(penAlignment);
//        return true;
//    }

    public void setFontName(JSONObject params) {
        String fontName = params.optString(PARAMS_FONT_NAME, "");
        if (!TextUtils.isEmpty(fontName)) {
            mApi.setDrawParam(IAtBitmap.DrawParamName.FONT_NAME, fontName);
        }
    }

    public void setBackground(JSONObject params) {
        this.mApi.setBackground(params.optInt(PARAMS_COLOR, Color.WHITE));
    }

    // ***************************************************
    // * 打印相关图形对象。
    // ***************************************************

    public void drawText(JSONObject params) {
        if (params == null)
            return;

        String text = params.optString(PARAMS_TEXT);
        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        double height = params.optDouble(PARAMS_HEIGHT);
        double fontHeight = params.optDouble(PARAMS_FONT_HEIGHT);
        int fontStyle = params.optInt(PARAMS_FONT_STYLE);
        int lineSpace = params.optInt(PARAMS_LINESPACE);
        String fontName = params.optString(PARAMS_FONT_NAME);
        if (TextUtils.isEmpty(text))
            return;
        if (!TextUtils.isEmpty(fontName)) {
            this.mApi.setDrawParam(IAtBitmap.DrawParamName.FONT_NAME, fontName);
        }
        //
        int oldRotation = mApi.getItemOrientation();
        int oldHorAlign = mApi.getItemHorizontalAlignment();
        int oldVerAlign = mApi.getItemVerticalAlignment();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);
        int horAlign = params.optInt("horizontalAlignment", oldHorAlign);
        int verAlign = params.optInt("verticalAlignment", oldVerAlign);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(rotation);
        if (horAlign != oldHorAlign)
            mApi.setItemHorizontalAlignment(horAlign);
        if (verAlign != oldVerAlign)
            mApi.setItemVerticalAlignment(verAlign);
        //
        this.mApi.drawTextRegular(text, x, y, width, height, fontHeight, fontStyle, lineSpace);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(oldRotation);
        if (horAlign != oldHorAlign)
            mApi.setItemHorizontalAlignment(oldHorAlign);
        if (verAlign != oldVerAlign)
            mApi.setItemVerticalAlignment(oldVerAlign);
    }

    public void draw1DBarcode(JSONObject params) {
        if (params == null)
            return;

        String text = params.optString(PARAMS_TEXT);
        int type = params.optInt(PARAMS_TYPE);
        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        double height = params.optDouble(PARAMS_HEIGHT);
        double textHeight = params.optDouble(PARAMS_TEXT_HEIGHT);
        if (TextUtils.isEmpty(text))
            return;
        //
        int oldRotation = mApi.getItemOrientation();
        int oldHorAlign = mApi.getItemHorizontalAlignment();
        int oldVerAlign = mApi.getItemVerticalAlignment();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);
        int horAlign = params.optInt("horizontalAlignment", -1);
        int verAlign = params.optInt("verticalAlignment", -1);
        double minScale = params.optDouble("minScaleLevel", -1);
        boolean useDefaultScaleLevel = minScale < 0 || minScale >= 2;
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(rotation);
        if (horAlign >= 0 && horAlign != oldHorAlign)
            mApi.setItemHorizontalAlignment(horAlign);
        if (verAlign >= 0 && verAlign != oldVerAlign)
            mApi.setItemVerticalAlignment(verAlign);
        //
        this.mApi.draw1DBarcode(text, type, x, y, width, height, textHeight, useDefaultScaleLevel);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(oldRotation);
        if (horAlign >= 0 && horAlign != oldHorAlign)
            mApi.setItemHorizontalAlignment(oldHorAlign);
        if (verAlign >= 0 && verAlign != oldVerAlign)
            mApi.setItemVerticalAlignment(oldVerAlign);
    }

    public void draw2DQRCode(JSONObject params) {
        if (params == null)
            return;

        String text = params.optString(PARAMS_TEXT);
        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        if (TextUtils.isEmpty(text))
            return;
        //
        int oldRotation = mApi.getItemOrientation();
        int oldHorAlign = mApi.getItemHorizontalAlignment();
        int oldVerAlign = mApi.getItemVerticalAlignment();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);
        int horAlign = params.optInt("horizontalAlignment", oldHorAlign);
        int verAlign = params.optInt("verticalAlignment", oldVerAlign);
        int eccLevel = params.optInt(PARAMS_ECC_LEVEL, -1);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(rotation);
        if (horAlign != oldHorAlign)
            mApi.setItemHorizontalAlignment(horAlign);
        if (verAlign != oldVerAlign)
            mApi.setItemVerticalAlignment(verAlign);
        if (eccLevel >= 0)
            mApi.setDrawParam(IAtBitmap.DrawParamName.ERROR_CORRECTION, eccLevel);
        //
        this.mApi.draw2DQRCode(text, x, y, width);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(oldRotation);
        if (horAlign != oldHorAlign)
            mApi.setItemHorizontalAlignment(oldHorAlign);
        if (verAlign != oldVerAlign)
            mApi.setItemVerticalAlignment(oldVerAlign);
    }

//    public void draw2DPdf417(JSONObject params) {
//        if (params == null)
//            return;
//
//        String text = params.optString(PARAMS_TEXT);
//        double x = params.optDouble(PARAMS_X);
//        double y = params.optDouble(PARAMS_Y);
//        double width = params.optDouble(PARAMS_WIDTH);
//        double height = params.optDouble(PARAMS_HEIGHT);
//        if (TextUtils.isEmpty(text))
//            return;
//
//        this.mApi.draw2DPdf417(text, x, y, width, height);
//    }

    public void drawRectangle(JSONObject params) {
        if (params == null)
            return;

        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        double height = params.optDouble(PARAMS_HEIGHT);
        double lineWidth = params.optDouble(PARAMS_LINE_WIDTH, LINEWIDTH_DEFAULT);
        double cornerWidth = params.optDouble(PARAMS_CORNER_WIDTH);
        boolean fill = params.optBoolean("fill", false);
        //
        int oldRotation = mApi.getItemOrientation();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(rotation);
        if (fill) {
            if (cornerWidth > 0)
                mApi.fillRoundRectangle(x, y, width, height, cornerWidth, cornerWidth);
            else
                mApi.fillRectangle(x, y, width, height);
        } else {
            if (cornerWidth > 0)
                mApi.drawRoundRectangle(x, y, width, height, cornerWidth, cornerWidth, lineWidth);
            else
                mApi.drawRectangle(x, y, width, height, lineWidth);
        }
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(oldRotation);
    }

    public void fillRectangle(JSONObject params) {
        if (params == null)
            return;
        //
        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        double height = params.optDouble(PARAMS_HEIGHT);
        //
        mApi.fillRectangle(x, y, width, height);
    }

    public void drawRoundRectangle(JSONObject params) {
        if (params == null)
            return;
        //
        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        double height = params.optDouble(PARAMS_HEIGHT);
        double corner = params.optDouble(PARAMS_CORNER, CORNER_DEFAULT);
        double cornerWidth = params.optDouble(PARAMS_CORNER_WIDTH, corner);
        double cornerHeight = params.optDouble(PARAMS_CORNER_HEIGHT, corner > 0 ? corner : cornerWidth);
        double lineWidth = params.optDouble(PARAMS_LINE_WIDTH, LINEWIDTH_DEFAULT);
        boolean fill = params.optBoolean("fill");
        int oldRotation = mApi.getItemOrientation();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(rotation);
        if (fill) {
            mApi.fillRoundRectangle(x, y, width, height, cornerWidth, cornerHeight);
        } else {
            mApi.drawRoundRectangle(x, y, width, height, cornerWidth, cornerHeight, lineWidth);
        }
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(oldRotation);
    }

    public void fillRoundRectangle(JSONObject params) {
        if (params == null)
            return;

        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        double height = params.optDouble(PARAMS_HEIGHT);
        double corner = params.optDouble(PARAMS_CORNER, CORNER_DEFAULT);
        double cornerWidth = params.optDouble(PARAMS_CORNER_WIDTH, corner);
        double cornerHeight = params.optDouble(PARAMS_CORNER_HEIGHT, corner);
        int oldRotation = mApi.getItemOrientation();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(rotation);
        //
        mApi.fillRoundRectangle(x, y, width, height, cornerWidth, cornerHeight);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(oldRotation);
    }

    public void drawEllipse(JSONObject params) {
        if (params == null)
            return;

        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        double height = params.optDouble(PARAMS_HEIGHT);
        double lineWidth = params.optDouble(PARAMS_LINE_WIDTH, LINEWIDTH_DEFAULT);
        boolean fill = params.optBoolean("fill");
        int oldRotation = mApi.getItemOrientation();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(rotation);
        if (fill) {
            mApi.fillEllipse(x, y, width, height);
        } else {
            mApi.drawEllipse(x, y, width, height, lineWidth);
        }
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(oldRotation);
    }

    public void fillEllipse(JSONObject params) {
        if (params == null)
            return;
        //
        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        double height = params.optDouble(PARAMS_HEIGHT);
        int oldRotation = mApi.getItemOrientation();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(rotation);
        //
        mApi.fillEllipse(x, y, width, height);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(oldRotation);
    }

    public void drawCircle(JSONObject params) {
        if (params == null)
            return;

        double x = params.optDouble(PARAMS_X, 0);
        double y = params.optDouble(PARAMS_Y, 0);
        double radius = params.optDouble(PARAMS_RADIUS, 0);
        double width = params.optDouble(PARAMS_WIDTH, 0);
        double height = params.optDouble(PARAMS_HEIGHT, 0);
        double lineWidth = params.optDouble(PARAMS_LINE_WIDTH, LINEWIDTH_DEFAULT);
        boolean fill = params.optBoolean("fill");
        //
        if (radius <= 0) {
            if (width > 0 && height > 0) {
                x += width * 0.5;
                y += height * 0.5;
                radius = Math.min(width, height) * 0.5;
            } else if (width > 0) {
                x += width * 0.5;
                radius = width * 0.5;
            } else if (height > 0) {
                y += height * 0.5;
                radius = height * 0.5;
            }
        }
        //
        if (fill) {
            mApi.fillCircle(x, y, radius);
        } else {
            mApi.drawCircle(x, y, radius, lineWidth);
        }
    }

    public void fillCircle(JSONObject params) {
        if (params == null)
            return;

        double x = params.optDouble(PARAMS_X, 0);
        double y = params.optDouble(PARAMS_Y, 0);
        double radius = params.optDouble(PARAMS_RADIUS, 0);
        double width = params.optDouble(PARAMS_WIDTH, 0);
        double height = params.optDouble(PARAMS_HEIGHT, 0);
        //
        if (radius <= 0) {
            if (width > 0 && height > 0) {
                x += width * 0.5;
                y += height * 0.5;
                radius = Math.min(width, height) * 0.5;
            } else if (width > 0) {
                x += width * 0.5;
                radius = width * 0.5;
            } else if (height > 0) {
                y += height * 0.5;
                radius = height * 0.5;
            }
        }
        //
        mApi.fillCircle(x, y, radius);
    }

    public void drawLine(JSONObject params) {
        if (params == null)
            return;

        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH, 0);
        double height = params.optDouble(PARAMS_HEIGHT, 0);
        double x1 = params.optDouble(PARAMS_X1, x);
        double y1 = params.optDouble(PARAMS_Y1, y);
        double x2 = params.optDouble(PARAMS_X2, width > 0 ? x1 + width : x1);
        double y2 = params.optDouble(PARAMS_Y2, height > 0 ? y1 + height : y1);
        double lineWidth = params.optDouble(PARAMS_LINE_WIDTH, LINEWIDTH_DEFAULT);
        JSONArray dashLens = params.optJSONArray("dashLens");
        //
        if (width > 0 && height > 0) {
            // 在同时制定了宽高的时候，相关位置信息做下处理，否则有可能直线变斜线
            if (width > height) {
                // 绘制横线
                y1 += height * 0.5;
                y2 = y1;
                x2 = x1 + width;
                lineWidth = height;
            } else {
                // 绘制竖线
                x1 += width * 0.5;
                x2 = x1;
                y2 = y1 + height;
                lineWidth = width;
            }
        }
        if (dashLens == null)
            dashLens = params.optJSONArray(PARAMS_DASH_LEN);
        //
        int oldRotation = mApi.getItemOrientation();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(rotation);
        //
        if (dashLens != null && dashLens.length() > 0) {
            List<Double> list = new ArrayList<>();
            for (int i = 0; i < dashLens.length(); i++) {
                list.add(dashLens.optDouble(i));
            }
            if (list.size() < 2) list.add(list.get(0));
            double[] arr = new double[list.size()];
            for (int i = 0; i < list.size(); i++) {
                arr[i] = list.get(i);
            }
            mApi.drawDashLine(x1, y1, x2, y2, lineWidth, arr, arr.length);
        } else {
            mApi.drawLine(x1, y1, x2, y2, lineWidth);
        }
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(oldRotation);
    }

    public void drawDashLine(JSONObject params) {
        if (params == null)
            return;
        //
        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        double height = params.optDouble(PARAMS_HEIGHT);
        double x1 = params.optDouble(PARAMS_X1, x);
        double y1 = params.optDouble(PARAMS_Y1, y);
        double x2 = params.optDouble(PARAMS_X2, width > 0 ? x1 + width : x1);
        double y2 = params.optDouble(PARAMS_Y2, height > 0 ? y1 + height : y1);
        double lineWidth = params.optDouble(PARAMS_LINE_WIDTH, LINEWIDTH_DEFAULT);
        int oldRotation = mApi.getItemOrientation();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);

        try {
            List<Double> dashLen = new ArrayList<>();
            JSONArray asDashLen = params.optJSONArray(PARAMS_DASH_LEN);
            if (asDashLen != null && asDashLen.length() > 0) {
                for (int i = 0; i < asDashLen.length(); i++) {
                    dashLen.add(asDashLen.optDouble(i));
                }
            }
            if (dashLen.size() <= 0) {
                dashLen.add(1.0);
                dashLen.add(1.0);
            }

            double[] dDashLen = new double[dashLen.size()];
            for (int i = 0; i < dashLen.size(); i++) {
                dDashLen[i] = dashLen.get(i);
            }
            //
            if (rotation != oldRotation)
                mApi.setItemOrientation(rotation);
            //
            mApi.drawDashLine(x1, y1, x2, y2, lineWidth, dDashLen, dDashLen.length);
            //
            if (rotation != oldRotation)
                mApi.setItemOrientation(oldRotation);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void drawImage(JSONObject params) {
        if (params == null)
            return;

        String image = params.optString(PARAMS_IMAGE);
        if (TextUtils.isEmpty(image))
            image = params.optString("data");
        if (TextUtils.isEmpty(image))
            return;
        //
        double x = params.optDouble(PARAMS_X);
        double y = params.optDouble(PARAMS_Y);
        double width = params.optDouble(PARAMS_WIDTH);
        double height = params.optDouble(PARAMS_HEIGHT);
        int threshold = params.optInt(PARAMS_THRESHOLD, THRESHOLD_DEFAULT);
        threshold = threshold > 0 ? threshold : 192;
        //
        int oldRotation = mApi.getItemOrientation();
        int oldHorAlign = mApi.getItemHorizontalAlignment();
        int oldVerAlign = mApi.getItemVerticalAlignment();
        int rotation = params.optInt(PARAMS_ORIENTATION, oldRotation);
        int horAlign = params.optInt("horizontalAlignment", oldHorAlign);
        int verAlign = params.optInt("verticalAlignment", oldVerAlign);
        //
        if (rotation != oldRotation)
            mApi.setItemOrientation(rotation);
        if (horAlign != oldHorAlign)
            mApi.setItemHorizontalAlignment(horAlign);
        if (verAlign != oldVerAlign)
            mApi.setItemVerticalAlignment(verAlign);
        //
        Bitmap bmp = decodeBase64(image);
        if (bmp != null)
            this.mApi.drawBitmapWithThreshold(decodeBase64(image), x, y, width, height, threshold);
        else
            this.mApi.drawImageWithThreshold(image, x, y, width, height, threshold);
        // 恢复对齐和旋转状态
        if (rotation != oldRotation)
            mApi.setItemOrientation(oldRotation);
        if (horAlign != oldHorAlign)
            mApi.setItemHorizontalAlignment(oldHorAlign);
        if (verAlign != oldVerAlign)
            mApi.setItemVerticalAlignment(oldVerAlign);
    }

//    public static boolean isBase64Image(String image) {
//        if (TextUtils.isEmpty(image))
//            return false;
//
//        return image.matches(REG_IMG_BASE64);
//    }

    /**
     * 将给定的bitmap对象转换成base64字符串；
     */
    public static String encodeBase64(Bitmap bmp) {
        if (bmp == null) {
            return null;
        }

        try (ByteArrayOutputStream baos = new ByteArrayOutputStream()) {
            bmp.compress(Bitmap.CompressFormat.PNG, 100, baos);
            baos.flush();

            return "data:image/png;base64," + Base64.encodeToString(baos.toByteArray(), Base64.DEFAULT);
        } catch (IOException e) {
            e.printStackTrace();
            return "";
        }
    }

    public static byte[] base64ToByteArray(String data) {
        if (TextUtils.isEmpty(data)) {
            return null;
        }
        try {
            return Base64.decode(data, Base64.DEFAULT);
        } catch (Exception e) {
            return null;
        }
    }

    public static String byteArrayToBase64(byte[] bytes) {
        if (bytes != null && bytes.length > 0) {
            return Base64.encodeToString(bytes, Base64.DEFAULT);
        } else {
            return "";
        }
    }

    public static Bitmap decodeBase64(String content) {
        if (TextUtils.isEmpty(content)) return null;
        //
        String data = "";
        if (content.indexOf(',') > 0) {
            data = content.split(",")[1];
        }
        //
        if (TextUtils.isEmpty(data)) {
            return null;
        }

        try {
            byte[] decode = Base64.decode(data, Base64.DEFAULT);
            return BitmapFactory.decodeByteArray(decode, 0, decode.length);
        } catch (Exception e) {
            return null;
        }
    }
}
