package com.dothantech.lpapi.cordova;

import android.Manifest;
import android.content.pm.PackageManager;
import android.os.Build;
import android.text.TextUtils;
import android.util.Log;

import com.dothantech.lpapi.LPAPI;
import com.dothantech.printer.IDzPrinter;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.apache.cordova.PermissionHelper;
import org.apache.cordova.PluginResult;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class LPAPIPlugin extends CordovaPlugin {
    private static final String TAG = "CordovaPlugin";

    private static final String ACTION_REQUEST_AUTH = "requestBluetoothAuth";
    private static final String ACTION_GET_PRINTERS = "getAllPrinters";
    private static final String ACTION_GET_FIRST_PRINTER = "getFirstPrinter";
    private static final String ACTION_GET_PRINTER_NAME = "getPrinterName";
    private static final String ACTION_GET_PRINTER_STATE = "getPrinterState";
    private static final String ACTION_GET_PRINTER_INFO = "getPrinterInfo";
    private static final String ACTION_IS_PRINTER_OPENED = "isPrinterOpened";
    private static final String ACTION_OPEN_PRINTER = "openPrinter";
    private static final String ACTION_CLOSE_PRINTER = "closePrinter";
    private static final String ACTION_GET_HORIZONTAL_ALIGNMENT = "getItemHorizontalAlignment";
    private static final String ACTION_SET_HORIZONTAL_ALIGNMENT = "setItemHorizontalAlignment";
    private static final String ACTION_GET_VERTICAL_ALIGNMENT = "getItemVerticalAlignment";
    private static final String ACTION_SET_VERTICAL_ALIGNMENT = "setItemVerticalAlignment";
    private static final String ACTION_GET_ORIENTATION = "getItemOrientation";
    private static final String ACTION_SET_ORIENTATION = "setItemOrientation";
    private static final String ACTION_SET_DRAW_PARAM = "setDrawParam";
    private static final String ACTION_SET_BACKGROUND = "setBackground";
    private static final String ACTION_START_JOB = "startJob";
    private static final String ACTION_END_JOB = "endJob";
    private static final String ACTION_COMMIT_JOB = "commitJob";
    private static final String ACTION_ABORT_JOB = "abortJob";
    private static final String ACTION_START_PAGE = "startPage";
    private static final String ACTION_END_PAGE = "endPage";
    private static final String ACTION_GET_JOB_PAGES = "getJobPages";
    private static final String ACTION_PRINT_IMAGE = "printImage";
    private static final String ACTION_CANCEL = "cancel";
    //
    private static final String ACTION_DRAW_TEXT = "drawText";
    private static final String ACTION_DRAW_BARCODE = "draw1DBarcode";
    private static final String ACTION_DRAW_QRCODE = "draw2DQRCode";
    private static final String ACTION_DRAW_PDF417 = "draw2DPdf417";
    private static final String ACTION_DRAW_LINE = "drawLine";
    private static final String ACTION_DRAW_DASH_LINE = "drawDashLine";
    private static final String ACTION_DRAW_RECTANGLE = "drawRectangle";
    private static final String ACTION_DRAW_ROUND_RECTANGLE = "drawRoundRectangle";
    private static final String ACTION_DRAW_ELLIPSE = "drawEllipse";
    private static final String ACTION_DRAW_CIRCLE = "drawCircle";
    private static final String ACTION_FILL_RECTANGLE = "fillRectangle";
    private static final String ACTION_FILL_ROUND_RECTANGLE = "fillRoundRectangle";
    private static final String ACTION_FILL_ELLIPSE = "fillEllipse";
    private static final String ACTION_FILL_CIRCLE = "fillCircle";
    private static final String ACTION_DRAW_IMAGE = "drawImage";

    private LPAPIBridge bridge;
    private PermissionRequestResult mPermissionRequestCallback;

    private static final int REQUEST_CODE_AUTH = 0x991;

    public enum LPA_Result {
        OK,
        ERROR_PARAM,
        ERROR_SEND_FAILED,
        ERROR_TIMEOUT,
    }

    /**
     * 权限申请回调函数。
     */
    public interface PermissionRequestResult {
        /**
         * 权限申请结果。
         *
         * @param success    权限申请成功与否
         * @param permission 当权限申请失败的时候，参数为失败的权限名称。
         */
        void complete(boolean success, String permission);
    }

    public static JSONObject getJsonResult(int statusCode, Object resultInfo) {
        JSONObject obj = new JSONObject();
        try {
            obj.put("statusCode", statusCode);
            if (resultInfo != null) {
                obj.put("resultInfo", resultInfo);
            }
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
        return obj;
    }

    public static JSONObject getJsonResult(int statusCode) {
        return getJsonResult(statusCode, null);
    }

    /**
     * 判断系统是否已经获取了所有指定的权限。
     */
    private boolean haveAllPermissions(String[] permissions) {
        for (String permission : permissions) {
            if (!PermissionHelper.hasPermission(this, permission)) {
                return false;
            }
        }
        return true;
    }

    private void checkPermission(PermissionRequestResult callback) {
        List<String> permissions = new ArrayList<>();
        permissions.add(Manifest.permission.BLUETOOTH);
        permissions.add(Manifest.permission.BLUETOOTH_ADMIN);
        permissions.add(Manifest.permission.ACCESS_FINE_LOCATION);
        permissions.add(Manifest.permission.ACCESS_COARSE_LOCATION);
        if (Build.VERSION.SDK_INT > 30) {
            permissions.add(Manifest.permission.BLUETOOTH_SCAN);
            permissions.add(Manifest.permission.BLUETOOTH_CONNECT);
        }
        //
        String[] items = permissions.toArray(new String[0]);
        if (haveAllPermissions(items)) {
            callback.complete(true, null);
            return;
        }
        //
        this.mPermissionRequestCallback = callback;
        PermissionHelper.requestPermissions(this, REQUEST_CODE_AUTH, items);
    }

//    @Override
//    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) throws JSONException {
//        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
//        // 该回调韩式一直没有触发，不知道为什么
//        Log.i(TAG, "---- onRequestPermission【s】Result：");
//        if (requestCode == REQUEST_CODE_AUTH) {
//            for (int i = 0; i < grantResults.length; i++) {
//                if (grantResults[i] == PackageManager.PERMISSION_DENIED) {
//                    if (mPermissionRequestCallback != null) {
//                        mPermissionRequestCallback.complete(false, permissions[i]);
//                        mPermissionRequestCallback = null;
//                        return;
//                    }
//                }
//            }
//            // 权限申请成功
//            Log.i(TAG, "蓝牙权限申请成功！");
//            if (mPermissionRequestCallback != null) {
//                mPermissionRequestCallback.complete(true, null);
//                mPermissionRequestCallback = null;
//            }
//        }
//    }

    @Override
    public void onRequestPermissionResult(int requestCode, String[] permissions, int[] grantResults) throws JSONException {
        super.onRequestPermissionResult(requestCode, permissions, grantResults);
        //
        Log.i(TAG, "---- onRequestPermissionResult [deprecated]：");
        if (requestCode == REQUEST_CODE_AUTH) {
            for (int i = 0; i < grantResults.length; i++) {
                if (grantResults[i] == PackageManager.PERMISSION_DENIED) {
                    if (mPermissionRequestCallback != null) {
                        mPermissionRequestCallback.complete(false, permissions[i]);
                        mPermissionRequestCallback = null;
                        return;
                    }
                }
            }
            // 权限申请成功
            Log.i(TAG, "蓝牙权限申请成功！");
            if (mPermissionRequestCallback != null) {
                mPermissionRequestCallback.complete(true, null);
                mPermissionRequestCallback = null;
            }
        }
    }

    @Override
    public void initialize(CordovaInterface cordova, CordovaWebView webView) {
        super.initialize(cordova, webView);
        this.bridge = LPAPIBridge.getInstance();
        //
        Log.i("test", "initialize");
    }

    private void initPlugin(JSONObject options, final CallbackContext callback) {
        int clientType = options.optInt(LPAPIBridge.CLIENT_TYPE, -1);
        String models = options.optString(LPAPIBridge.MODELS, "");
        boolean autoDisconnect = options.optBoolean("autoDisconnect", true);
        boolean bleMode = options.optBoolean("bleMode", false);
        if (!bleMode)
            bleMode = options.optBoolean("ble", false);
        if (clientType >= 0) {
            bridge.api().setCT((byte) clientType);
        }
        if (!TextUtils.isEmpty(models)) {
            bridge.setPrinterModels(models);
        }
        bridge.api().supportPreferBluetoothBLE(bleMode);
        // 息屏的时候是否自动关闭蓝牙设备？
        IDzPrinter.Factory.enableAutoDisconnect(autoDisconnect);
        // 为了简化操作，初始化的时候可以动态申请下蓝牙权限
        this.checkPermission((success, permission) -> {
            if (success) {
                if (callback != null) callback.success();
            } else {
                if (callback != null) callback.error("");
            }
        });
    }

    @Override
    public boolean execute(String action, String args, final CallbackContext callback) throws JSONException {
        JSONObject params = (new JSONObject(args == null ? "" : args));
        Log.i(TAG, String.format("---- execute.action = %s", action));
        Log.i(TAG, args == null ? "undefined" : args);
        switch (action) {
            case "init":
            case "initPlugin":
                this.initPlugin(params, callback);
                return true;
            case ACTION_REQUEST_AUTH:
                this.requestBluetoothAuth(params, callback);
                return true;
            case "setPrinterModels":
                this.setPrinterModels(params, callback);
                return true;
            case "isPrinterSupported":
                this.isPrinterSupported(params, callback);
                return true;
            case "startScan":
            case "startBleDiscovery":
                this.startBleDiscovery(params, callback);
                return true;
            case "stopScan":
            case "stopBleDiscovery":
                this.stopBleDiscovery(params, callback);
                return true;
            case "writeCommand":
            case "writeCommands":
                this.writeCommands(params, callback);
                return true;
            case ACTION_GET_PRINTERS:
                this.getPrinters(params, callback);
                return true;
            case ACTION_GET_FIRST_PRINTER:
                this.getFirstPrinter(params, callback);
                return true;
            case ACTION_OPEN_PRINTER:
                this.openPrinter(params, callback);
                return true;
            case ACTION_GET_PRINTER_NAME:
                this.getPrinterName(params, callback);
                return true;
            case ACTION_GET_PRINTER_INFO:
                this.getPrinterInfo(params, callback);
                return true;
            case ACTION_GET_PRINTER_STATE:
                this.getPrinterState(params, callback);
                return true;
            case ACTION_IS_PRINTER_OPENED:
                this.isPrinterOpened(params, callback);
                return true;
            case ACTION_CANCEL:
                this.cancel(params, callback);
                return true;
            case ACTION_CLOSE_PRINTER:
                this.closePrinter(params, callback);
                return true;
            case ACTION_PRINT_IMAGE:
                this.printImage(params, callback);
                return true;
            case ACTION_START_JOB:
                this.startJob(params, callback);
                return true;
            case ACTION_ABORT_JOB:
                this.abortJob(params, callback);
                return true;
            case ACTION_END_JOB:
                this.endJob(params, callback);
                return true;
            case ACTION_COMMIT_JOB:
                this.commitJob(params, callback);
                return true;
            case ACTION_START_PAGE:
                this.startPage(params, callback);
                return true;
            case ACTION_END_PAGE:
                this.endPage(params, callback);
                return true;
            case ACTION_GET_JOB_PAGES:
                this.getJobPages(params, callback);
                return true;
            case ACTION_SET_DRAW_PARAM:
                this.setDrawParam(params, callback);
                return true;
            case "setFontName":
                this.setFontName(params, callback);
                return true;
            case ACTION_SET_BACKGROUND:
                this.setBackground(params, callback);
                return true;
            case ACTION_GET_HORIZONTAL_ALIGNMENT:
                this.getItemHorizontalAlignment(params, callback);
                return true;
            case ACTION_SET_HORIZONTAL_ALIGNMENT:
                this.setItemHorizontalAlignment(params, callback);
                return true;
            case ACTION_GET_VERTICAL_ALIGNMENT:
                this.getItemVerticalAlignment(params, callback);
                return true;
            case ACTION_SET_VERTICAL_ALIGNMENT:
                this.setItemVerticalAlignment(params, callback);
                return true;
            case ACTION_GET_ORIENTATION:
                this.getItemOrientation(params, callback);
                return true;
            case ACTION_SET_ORIENTATION:
                this.setItemOrientation(params, callback);
                return true;
            case ACTION_DRAW_TEXT:
                this.drawText(params, callback);
                return true;
            case ACTION_DRAW_BARCODE:
            case "drawBarcode":
                this.draw1DBarcode(params, callback);
                return true;
            case ACTION_DRAW_QRCODE:
            case "drawQRCode":
                this.draw2DQRCode(params, callback);
                return true;
            case ACTION_DRAW_PDF417:
            case "drawPdf417":
            case "drawPDF417":
                this.draw2DPdf417(params, callback);
                return true;
            case ACTION_DRAW_RECTANGLE:
                this.drawRectangle(params, callback);
                return true;
            case ACTION_FILL_RECTANGLE:
                this.fillRectangle(params, callback);
                return true;
            case ACTION_DRAW_ROUND_RECTANGLE:
                this.drawRoundRectangle(params, callback);
                return true;
            case ACTION_FILL_ROUND_RECTANGLE:
                this.fillRoundRectangle(params, callback);
                return true;
            case ACTION_DRAW_ELLIPSE:
                this.drawEllipse(params, callback);
                return true;
            case ACTION_FILL_ELLIPSE:
                this.fillEllipse(params, callback);
                return true;
            case ACTION_DRAW_CIRCLE:
                this.drawCircle(params, callback);
                return true;
            case ACTION_FILL_CIRCLE:
                this.fillCircle(params, callback);
                return true;
            case ACTION_DRAW_LINE:
                this.drawLine(params, callback);
                return true;
            case ACTION_DRAW_DASH_LINE:
                this.drawDashLine(params, callback);
                return true;
            case ACTION_DRAW_IMAGE:
                this.drawImage(params, callback);
                return true;
            default:
                break;
        }

        return super.execute(action, args, callback);
    }

    //////////////////////////////////////
    ///     Printer interface
    /////////////////////////////////////

    public void requestBluetoothAuth(JSONObject options, CallbackContext callback) {
        this.checkPermission((success, permission) -> {
            if (success) {
                if (callback != null) callback.success();
            } else {
                if (callback != null) callback.error("");
            }
        });
    }

    public void setPrinterModels(JSONObject options, CallbackContext callback) {
        String models = options.optString(LPAPIBridge.MODELS);
        bridge.setPrinterModels(models);
        callback.success();
    }

    public void isPrinterSupported(JSONObject options, CallbackContext callback) {
        String deviceName = options.optString("name");
        String models = options.optString("models");
        boolean res = bridge.isPrinterSupported(deviceName, models);
        if (res) {
            callback.success();
        } else {
            callback.error("false");
        }
    }

    public void startBleDiscovery(JSONObject options, CallbackContext callback) {
        // 蓝牙搜索前，先检查蓝牙权限有没有打开，如果没有则申请蓝牙权限
        this.checkPermission((success, permission) -> {
            Log.i(TAG, String.format("---- checkPermission.callback, success = %b", success));
            if (success) {
                bridge.startBleDiscovery(options, (status, devices) -> {
                    if (status == LPAPIBridge.EDiscoveryStatus.OK) {
                        // 启动成功
                        JSONObject data = LPAPIPlugin.getJsonResult(status.ordinal(), null);
                        PluginResult pResult = new PluginResult(PluginResult.Status.OK, data);
                        pResult.setKeepCallback(true);
                        callback.sendPluginResult(pResult);
                    } else if (status == LPAPIBridge.EDiscoveryStatus.DEVICE_FOUND) {
                        if (devices != null && !devices.isEmpty()) {
                            JSONArray list = LPAPIBridge.getDeviceArray(devices);
                            JSONObject data = LPAPIPlugin.getJsonResult(status.ordinal(), list);
                            PluginResult pResult = new PluginResult(PluginResult.Status.OK, data);
                            pResult.setKeepCallback(true);
                            callback.sendPluginResult(pResult);
                        }
                    } else {
                        // 搜索完毕
                        Log.i(TAG, String.format("---- startBleDiscovery.complete, status = %d", status.ordinal()));
                        JSONObject data = getJsonResult(status.ordinal(), null);
                        PluginResult pResult = new PluginResult(PluginResult.Status.OK, data);
                        pResult.setKeepCallback(false);
                        callback.sendPluginResult(pResult);
                    }
                });
            } else {
                Log.i(TAG, "---- 蓝牙权限申请失败！");
                JSONObject data = getJsonResult(LPAPIBridge.EDiscoveryStatus.FAILED_PERMISSION.ordinal(), null);
                PluginResult pResult = new PluginResult(PluginResult.Status.OK, data);
                pResult.setKeepCallback(false);
                callback.sendPluginResult(pResult);
            }
        });
    }

    public void stopBleDiscovery(JSONObject options, CallbackContext callback) {
        bridge.stopBleDiscovery();
        callback.success();
    }

    public void getPrinters(JSONObject options, CallbackContext callback) {
        checkPermission(((success, permission) -> {
            JSONArray value = this.bridge.getAllPrinterAddresses(options);
            callback.success(value);
        }));
    }

    public void getFirstPrinter(JSONObject options, CallbackContext callback) {
        checkPermission(((success, permission) -> {
            JSONObject value = this.bridge.getFirstPrinter(options);
            callback.success(value);
        }));
    }

    public void openPrinter(final JSONObject options, final CallbackContext callback) {
        checkPermission(((success, permission) -> {
            boolean result = this.bridge.openPrinter(options, (success1, msg) -> {
                if (success1) {
                    callback.success(msg);
                } else {
                    // 重新连接的时候，通常会先断开旧的连接，然后再重新连接新的打印机；
                    PluginResult result1 = new PluginResult(PluginResult.Status.ERROR);
                    result1.setKeepCallback(true);
                    callback.sendPluginResult(result1);
                }
            });
            if (!result) {
                callback.error("参数错误！");
            }
        }));
    }

    public void getPrinterName(JSONObject options, CallbackContext callback) {
        String value = this.bridge.getPrinterName();
        callback.success(value);
    }

    public void getPrinterInfo(JSONObject options, CallbackContext callback) {
        JSONObject result = bridge.getPrinterInfo();
        callback.success(result);
    }

    public void getPrinterState(JSONObject options, CallbackContext callback) {
        JSONObject value = this.bridge.getPrinterState();
        callback.success(value);
    }

    public void isPrinterOpened(JSONObject options, CallbackContext callback) {
        boolean value = this.bridge.isPrinterOpened();
        if (value) {
            // success()默认返回“OK”;
            callback.success();
        } else {
            callback.error("打印机未连接");
        }
    }

    public void cancel(JSONObject options, CallbackContext callback) {
        bridge.cancel();
        callback.success();
    }

    public void closePrinter(JSONObject options, CallbackContext callback) {
        bridge.closePrinter();
        callback.success();
    }

    public void printImage(final JSONObject options, final CallbackContext callback) {
        boolean value = this.bridge.printImage(options, (success, msg) -> {
            if (success) {
                callback.success(msg);
            } else {
                callback.error(msg);
            }
        });
        if (!value) {
            callback.error("请求失败！");
        }
    }

    private static String toHexString(byte[] data, String prefix, String sep) {
        List<String> hexList = new ArrayList<>();
        if (prefix == null) prefix = "";
        if (sep == null) sep = "";
        for (byte datum : data) {
            hexList.add(String.format("%s%02x", prefix, datum));
        }
        return String.join(sep, hexList);
    }

    public void writeCommands(JSONObject options, final CallbackContext callback) {
        String value = "";
        if (options != null) {
            value = options.optString("value", "");
            if (TextUtils.isEmpty(value))
                value = options.optString("data", "");
        }
        if (TextUtils.isEmpty(value)) {
            callback.error(getJsonResult(LPA_Result.ERROR_PARAM.ordinal()));
            return;
        }
        // 将base64转换为
        byte[] buffer = LPAPIBridge.base64ToByteArray(value);
        if (buffer == null) {
            callback.error(getJsonResult(LPA_Result.ERROR_PARAM.ordinal()));
            return;
        }
        //
        // Log.i("writeCommands", "bytes = " + LPAPIPlugin.toHexString(buffer, "0x", ","));
        WriteCommandTask task = WriteCommandTask.createTask();
        LPAPI.Factory.command(buffer, new IDzPrinter.IActionWithResponseAllDataCallback() {

            @Override
            public void onProgress(IDzPrinter.GeneralProgress progress, Object o) {
//                Log.i(TAG, "------ onProgress, progress = " + progress.name());
                byte[] byteArray = task.respStream.toByteArray();
//                    Log.i(TAG, "respData = " + LPAPIPlugin.toHexString(byteArray, "0x", ","));
                String respData = LPAPIBridge.byteArrayToBase64(byteArray);
//                    Log.i("------", "respData = " + respData);
                if (progress == IDzPrinter.GeneralProgress.Success || progress == IDzPrinter.GeneralProgress.Success2) {
                    callback.success(getJsonResult(0, respData));
                } else if (progress == IDzPrinter.GeneralProgress.Timeout) {
                    callback.error(getJsonResult(LPA_Result.ERROR_TIMEOUT.ordinal(), respData));
                } else if (progress == IDzPrinter.GeneralProgress.Failed ||
                        progress == IDzPrinter.GeneralProgress.Cancelled) {
                    callback.error(getJsonResult(LPA_Result.ERROR_SEND_FAILED.ordinal()));
                }
            }

            @Override
            public void onResponse(byte[] bytes) {
                try {
                    task.respStream.write(bytes);
//                    Log.i(TAG, "writeCommands.onResponse: " + LPAPIPlugin.toHexString(bytes, "0x", ","));
                } catch (IOException e) {
                    Log.w(TAG, e.getMessage());
                }
            }
        });
    }

    public void startJob(JSONObject options, CallbackContext callback) {
        boolean value = this.bridge.startJob(options);
        if (value) {
            // 默认返回OK；
            callback.success();
        } else {
            callback.error("");
        }
    }

    public void abortJob(JSONObject options, CallbackContext callback) {
        bridge.abortJob();
        callback.success();
    }

    public void commitJob(JSONObject options, final CallbackContext callback) {
        boolean value = this.bridge.commitJob(options, (success, msg) -> {
            if (success) {
                callback.success(msg);
            } else {
                callback.error(msg);
            }
        });
        if (!value) {
            callback.error("请求失败");
        }
    }

    public void startPage(JSONObject options, CallbackContext callback) {
        boolean value = this.bridge.startPage();
        if (value) {
            callback.success();
        } else {
            callback.error("");
        }
    }

    public void endPage(JSONObject options, CallbackContext callback) {
        bridge.endPage();
        callback.success(0);
    }

    public void endJob(JSONObject options, CallbackContext callback) {
        bridge.endJob();
        callback.success(0);
    }

    public void getJobPages(JSONObject options, CallbackContext callback) {
        JSONArray value = this.bridge.getJobPages();
        callback.success(value);
    }

    //////////////////////////////////////
    ///     get/set params
    /////////////////////////////////////

    public void setDrawParam(JSONObject options, CallbackContext callback) {
        bridge.setDrawParam(options);
        callback.success(0);
    }

    public void getItemOrientation(JSONObject options, CallbackContext callback) {
        int value = this.bridge.getItemOrientation();
        callback.success(value);
    }

    public void setItemOrientation(JSONObject options, CallbackContext callback) {
        boolean value = this.bridge.setItemOrientation(options);
        if (value) {
            callback.success();
        } else {
            callback.error("");
        }
    }

    public void getItemHorizontalAlignment(JSONObject options, CallbackContext callback) {
        int value = this.bridge.getItemHorizontalAlignment();
        callback.success(value);
    }

    public void setItemHorizontalAlignment(JSONObject options, CallbackContext callback) {
        boolean value = this.bridge.setItemHorizontalAlignment(options);
        if (value) {
            callback.success();
        } else {
            callback.error("");
        }
    }

    public void getItemVerticalAlignment(JSONObject options, CallbackContext callback) {
        int value = bridge.getItemVerticalAlignment();
        callback.success(value);
    }

    public void setItemVerticalAlignment(JSONObject options, CallbackContext callback) {
        boolean value = this.bridge.setItemVerticalAlignment(options);
        if (value) {
            callback.success();
        } else {
            callback.error("");
        }
    }

    public void setFontName(JSONObject options, CallbackContext callback) {
        bridge.setFontName(options);
        callback.success();
    }

    public void setBackground(JSONObject options, CallbackContext callback) {
        bridge.setBackground(options);
        callback.success();
    }

    // ***************************************************
    // * 打印相关图形对象。
    // ***************************************************

    public void drawText(JSONObject options, CallbackContext callback) {
        bridge.drawText(options);
        callback.success();
    }

    public void draw1DBarcode(JSONObject options, CallbackContext callback) {
        bridge.draw1DBarcode(options);
        callback.success();
    }

    public void draw2DQRCode(JSONObject options, CallbackContext callback) {
        bridge.draw2DQRCode(options);
        callback.success();
    }

    public void draw2DPdf417(JSONObject options, CallbackContext callback) {
//        bridge.draw2DPdf417(options);
        bridge.draw2DQRCode(options);
        callback.success();
    }

    public void drawRectangle(JSONObject options, CallbackContext callback) {
        bridge.drawRectangle(options);
        callback.success();
    }

    public void fillRectangle(JSONObject options, CallbackContext callback) {
        bridge.fillRectangle(options);
        callback.success();
    }

    public void drawRoundRectangle(JSONObject options, CallbackContext callback) {
        bridge.drawRoundRectangle(options);
        callback.success();
    }

    public void fillRoundRectangle(JSONObject options, CallbackContext callback) {
        bridge.fillRoundRectangle(options);
        callback.success();
    }

    public void drawEllipse(JSONObject options, CallbackContext callback) {
        bridge.drawEllipse(options);
        callback.success();
    }

    public void fillEllipse(JSONObject options, CallbackContext callback) {
        bridge.fillEllipse(options);
        callback.success();
    }

    public void drawCircle(JSONObject options, CallbackContext callback) {
        bridge.drawCircle(options);
        callback.success();
    }

    public void fillCircle(JSONObject options, CallbackContext callback) {
        bridge.fillCircle(options);
        callback.success();
    }

    public void drawLine(JSONObject options, CallbackContext callback) {
        bridge.drawLine(options);
        callback.success();
    }

    public void drawDashLine(JSONObject options, CallbackContext callback) {
        bridge.drawDashLine(options);
        callback.success();
    }

    public void drawImage(JSONObject options, CallbackContext callback) {
        bridge.drawImage(options);
        callback.success();
    }
}
