//
//  AMLPAPI.m
//  LPAPICordovaTest
//
//  Created by 蔡俊杰 on 2017/12/14.
//

#import "AMLPAPI.h"

#import "LPAPI.h"

static BOOL isJobEnded = false;

@implementation AMLPAPI

@synthesize width;
@synthesize height;
@synthesize orientation;
@synthesize jobName;


- (BOOL)isCommandTrue:(CDVInvokedUrlCommand *)command
                count:(int)count
{
    if ([[command arguments] count] >= count)
    {
        return YES;
    }
    else
    {
        //如果没有入参,则回调JS失败函数
        CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR
                                                          messageAsString:@"没有参数"];
        [[self commandDelegate] sendPluginResult:pluginResult
                                      callbackId:[command callbackId]];
        
        return NO;
    }
}

-(void)sendPluginResult:(CDVInvokedUrlCommand *)command
                 status: (CDVCommandStatus) status
                 message:(NSString *) msg
{
    CDVPluginResult *result = [CDVPluginResult resultWithStatus:status messageAsString: msg == nil ? @"" : msg];
    [[self commandDelegate] sendPluginResult:result  callbackId:[command callbackId]];
}
-(void)sendSuccessResult:(CDVInvokedUrlCommand *)command
                 message:(NSString *) msg
{
    [self sendPluginResult:command status:CDVCommandStatus_OK message:msg == nil ? @"OK" : msg];
}
-(void)sendSuccessResult:(CDVInvokedUrlCommand *)command
                 withBoolMessage:(BOOL) msg
{
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsBool:msg];
    [[self commandDelegate] sendPluginResult:pluginResult callbackId:command.callbackId];
}
-(void)sendFailResult:(CDVInvokedUrlCommand *)command
                 message:(NSString *) msg
{
    [self sendPluginResult:command status:CDVCommandStatus_ERROR message:msg == nil ? @"ERROR" : msg];
}

-(BOOL)isPreviewJob:(NSString *)jobName
{
    if(jobName == nil)
        return false;
    //
    jobName = jobName.lowercaseString;
    return [jobName hasPrefix:@"#!#prev"] || [jobName hasPrefix:@"#!#trans"];
}

- (NSString *)encodeImage:(UIImage *) image
{
    if(image == nil)
    {
        NSLog(@"---- encodeImage: 参数不能为 nil!");
        return @"";
    }
    //
    NSData *imgData = UIImagePNGRepresentation(image);
//    return [imgData base64EncodedStringWithOptions:NSDataBase64Encoding64CharacterLineLength];
    NSString *dataUrl = [imgData base64EncodedStringWithOptions:0];
    return [NSString stringWithFormat:@"data:image/png;base64,%@", dataUrl];
}

- (UIImage *)decodeImage: (NSString *) dataSrc
{
    NSURL *url = [NSURL URLWithString: dataSrc];
    
    NSData *imgData = [NSData dataWithContentsOfURL:url];
    if(imgData == nil) {
        NSLog(@"---- 图片解析失败！[%@]", dataSrc);
        return nil;
    }
    //
    return [UIImage imageWithData:imgData];
}

- (void)initPlugin:(CDVInvokedUrlCommand *)command
{
    NSString *models = [command.arguments count] > 0 ? [command.arguments objectAtIndex:0] : @"";
    if(models != nil) {
        [LPAPI setSupportPrefixs:[models componentsSeparatedByString:@";"]];
    }
}

- (void)setSupportPrefixs:(CDVInvokedUrlCommand *)command
{
    NSString *models = [command.arguments count] > 0 ? [[command.arguments objectAtIndex:0] stringValue] : @"";
    if (models != nil)
    {
        [LPAPI setSupportPrefixs:[models componentsSeparatedByString:@";"]];
    }
}

- (void)connectingPrinterName:(CDVInvokedUrlCommand *)command
{
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                      messageAsString:[LPAPI connectingPrinterName]];
    [[self commandDelegate] sendPluginResult:pluginResult 
                                  callbackId:[command callbackId]];
}

- (void)scanPrinters:(CDVInvokedUrlCommand *)command
{
    NSString *models = [command.arguments count] > 0 ? [command.arguments objectAtIndex:0] : @"";
    if(models != nil && models.length > 0) {
        [LPAPI setSupportPrefixs:[models componentsSeparatedByString:@";"]];
    }
    [LPAPI scanPrinters:^(NSArray *scanedPrinterNames)
     {
         CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                            messageAsArray:scanedPrinterNames];
         [[self commandDelegate] sendPluginResult:pluginResult 
                                       callbackId:[command callbackId]];
     }];
}

- (void)openPrinter:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:1])
    {
        NSString *name = [command.arguments objectAtIndex:0];
        int clientType = [command.arguments count] > 1 ? [[command.arguments objectAtIndex:1] intValue] : -1;
        [LPAPI openPrinter:name
                completion:^(BOOL isSuccess)
         {
            if(isSuccess)
            {
                if(clientType >= 0)
                    [LPAPI setPrinterDeviceType:clientType];
                // 返回打印机详细信息
                [self getPrinterInfo:command];
            }else {
                [self sendFailResult:command message:@"ERROR_CONNECT_FAILED"];
            }
         }];
    } else {
        [self sendFailResult:command message:@"ERROR_PARAM"];
    }
}

-(void)getPrinterInfo:(CDVInvokedUrlCommand *)command
{
    PrinterInfo *info = [LPAPI connectingPrinterDetailInfos];
    NSMutableDictionary *dict = [NSMutableDictionary new];
    [dict setValue:info.deviceName forKey:@"deviceName"];
    [dict setValue:info.deviceAddress forKey:@"deviceAddress"];
    [dict setValue:info.deviceName forKey:@"name"];
    [dict setValue:info.deviceAddress forKey:@"mac"];
    [dict setValue:info.deviceAddress forKey:@"deviceId"];
    //
    [dict setValue:[NSNumber numberWithInt:info.deviceDPI] forKey:@"deviceDPI"];
    [dict setValue:[NSNumber numberWithInt:info.deviceWidth] forKey:@"deviceWidth"];
    [dict setValue:[NSNumber numberWithInt:info.deviceType] forKey:@"deviceType"];
    [dict setValue:info.deviceVersion forKey:@"deviceVersion"];
    [dict setValue:info.softwareVersion forKey:@"softwareVersion"];
    [dict setValue:info.manufacturer forKey:@"manufacturer"];
    [dict setValue:info.seriesName forKey:@"seriesName"];
    [dict setValue:info.devIntName forKey:@"devIntName"];
    [dict setValue:info.hardwareFlags forKey:@"hardwareFlags"];
    [dict setValue:[NSNumber numberWithInt:info.softwareFlags] forKey:@"softwareFlags"];
    [dict setValue:info.mcuID forKey:@"mcuId"];
    //
    CDVPluginResult *pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK
                                                       messageAsDictionary:dict];
    [[self commandDelegate] sendPluginResult:pluginResult
                                  callbackId:[command callbackId]];
}

- (void)closePrinter:(CDVInvokedUrlCommand *)command;
{
    [LPAPI closePrinter];
    [self sendSuccessResult:command withBoolMessage:true];
}

- (void)startPrint:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:2])
    {
        isJobEnded = false;
        width = [[command.arguments objectAtIndex:0] floatValue];
        height = [[command.arguments objectAtIndex:1] floatValue];
        orientation = [command.arguments count] > 2 ?  [[command.arguments objectAtIndex:2] intValue] : 0;
        // 打印任务名称
        jobName = [command.arguments count] > 3 ? [command.arguments objectAtIndex:3] : @"";
        NSInteger bgColor = [command.arguments count] > 4 ? [[command.arguments objectAtIndex:4] intValue] : -1;
        UIColor *color = [self isPreviewJob:jobName] && bgColor >= 0 ? nil : [UIColor whiteColor];
        //
        BOOL res = [LPAPI startDraw:width
                             height:height
                        orientation:orientation
                         background: color
        ];
        //
        [self sendSuccessResult:command withBoolMessage:res];
    } else {
        [self sendFailResult:command message:@"error_param"];
    }
}

- (void)endPrint:(CDVInvokedUrlCommand *)command
{
    if(isJobEnded) {
        [self sendSuccessResult:command withBoolMessage:true];
    } else {
        UIImage *img = [LPAPI endDraw];
        isJobEnded = true;
        //
        [self sendSuccessResult:command withBoolMessage:img != nil];
    }
}

- (void)previewImage:(CDVInvokedUrlCommand *)command
{
    UIImage *image = [LPAPI previewImage];
    NSString *dataUrl = [self encodeImage:(image)];
    if(image != nil) {
        [self sendSuccessResult:command message:dataUrl];
    } else {
        [self sendFailResult:command message:@"failed"];
    }
}

- (void)commitPrint:(CDVInvokedUrlCommand *)command
{
    if(!isJobEnded) {
        [LPAPI endDraw];
        isJobEnded = true;
    }
    int gapType = [command.arguments count] > 0 ? [[command.arguments objectAtIndex:0] intValue] : -1;
//    double gapLength = [command.arguments count] > 1 ? [[command.arguments objectAtIndex:1] floatValue] : 0;
    int printDarkness = [command.arguments count] > 2 ? [[command.arguments objectAtIndex:2] intValue] : -1;
    int printSpeed = [command.arguments count] > 3 ? [[command.arguments objectAtIndex:3] intValue] : -1;
//    int printCopies = [command.arguments count] > 4 ? [[command.arguments objectAtIndex:4] intValue] : 0;
    //
    NSMutableDictionary *dict = [NSMutableDictionary new];
    [dict setObject:[NSNumber numberWithDouble:width] forKey:@"width"];
    [dict setObject:[NSNumber numberWithDouble:height] forKey:@"height"];
    [dict setObject:[NSNumber numberWithInt:orientation] forKey:@"orientation"];
    [dict setObject:jobName forKey:@"jobName"];
    //
    if([self isPreviewJob:jobName]) {
        UIImage *image = [LPAPI previewImage];
        NSString *dataUrl = [self encodeImage:(image)];
        NSMutableArray<NSString*> *pages = [NSMutableArray new];
        CDVPluginResult *pluginResult = nil;
        if(image != nil) {
            [pages addObject:dataUrl];
            [dict setObject:pages forKey:@"pages"];
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK messageAsDictionary:dict];
        } else {
            pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsDictionary:dict];
        }
        [self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
    } else {
        if(gapType >= 0)
            [LPAPI setPrintPageGapType:gapType];
        if(printDarkness > 0)
            [LPAPI setPrintDarkness:printDarkness];
        if(printSpeed >= 0)
            [LPAPI setPrintSpeed:printSpeed];
        //
        [LPAPI print:^(BOOL isSuccess)
         {
            if(isSuccess) {
                [self sendSuccessResult:command withBoolMessage:true];
            } else {
                [self sendFailResult:command message:@"fail"];
            }
         }];
    }
}

- (void)setFontName:(CDVInvokedUrlCommand *)command
{
    //
    [self sendSuccessResult:command message:nil];
}
- (void)setItemOrientation:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:1])
    {
        [LPAPI setItemOrientation:[[[command arguments] objectAtIndex:0] intValue]];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)setItemHorizontalAlignment:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:1])
    {
        [LPAPI setItemHorizontalAlignment:[[[command arguments] objectAtIndex:0] intValue]];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)setItemVerticalAlignment:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:1])
    {
        [LPAPI setItemVerticalAlignment:[[[command arguments] objectAtIndex:0] intValue]];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawText:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:7])
    {
        int fontStyle = [command.arguments count] > 7 ? [[command.arguments objectAtIndex:7] intValue] : 0;
        float charSpace = [command.arguments count] > 8 ? [[command.arguments objectAtIndex:8] floatValue] : 0;
        float lineSpace = [command.arguments count] > 9 ? [[command.arguments objectAtIndex:9] floatValue] : 0;
        int rotation = [command.arguments count] > 10 ? [[command.arguments objectAtIndex:10] intValue] : -1;
        int horAlign = [command.arguments count] > 11 ? [[command.arguments objectAtIndex:11] intValue] : -1;
        int verAlign = [command.arguments count] > 12 ? [[command.arguments objectAtIndex:12] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        if(horAlign >= 0)
            [LPAPI setItemHorizontalAlignment:horAlign];
        if(verAlign >= 0)
            [LPAPI setItemVerticalAlignment:verAlign];
        //
        [LPAPI drawText:[[command arguments] objectAtIndex:0]
                      x:[[[command arguments] objectAtIndex:1] floatValue] 
                      y:[[[command arguments] objectAtIndex:2] floatValue]
                  width:[[[command arguments] objectAtIndex:3] floatValue]
                 height:[[[command arguments] objectAtIndex:4] floatValue]
               fontName:[[command arguments] objectAtIndex:5]
             fontHeight:[[[command arguments] objectAtIndex:6] floatValue]
              fontStyle:fontStyle
              charSpace:charSpace > 0 ? charSpace : 0
              lineSpace:lineSpace > 0 ? lineSpace : 0];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawBarcode:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:9])
    {
        int type = [command.arguments count] > 9 ? [[command.arguments objectAtIndex:9] intValue] : 0;
        int rotation = [command.arguments count] > 10 ? [[command.arguments objectAtIndex:10] intValue] : -1;
        int horAlign = [command.arguments count] > 11 ? [[command.arguments objectAtIndex:11] intValue] : -1;
        int verAlign = [command.arguments count] > 12 ? [[command.arguments objectAtIndex:12] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        if(horAlign >= 0)
            [LPAPI setItemHorizontalAlignment:horAlign];
        if(verAlign >= 0)
            [LPAPI setItemVerticalAlignment:verAlign];
        //
        [LPAPI drawBarcode:[[command arguments] objectAtIndex:0]
                         x:[[[command arguments] objectAtIndex:1] floatValue] 
                         y:[[[command arguments] objectAtIndex:2] floatValue]
                     width:[[[command arguments] objectAtIndex:3] floatValue]
                    height:[[[command arguments] objectAtIndex:4] floatValue]
                  fontName:[[command arguments] objectAtIndex:5]
                fontHeight:[[[command arguments] objectAtIndex:6] floatValue]
                 fontStyle:[[[command arguments] objectAtIndex:7] intValue]
                  textFlag:[[[command arguments] objectAtIndex:8] intValue]
                encodeMode: type >= 20 ? type : DzBarcodeEncodeMode_Code128
            autoResetWidth:YES];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawQRCode:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:4])
    {
        float width = [[command.arguments objectAtIndex:3] floatValue];
        float height = [command.arguments count] > 4 ? [[command.arguments objectAtIndex:4] floatValue] : width;
        int eccLevel = [command.arguments count] > 5 ? [[command.arguments objectAtIndex:5] intValue] : -1;
        int rotation = [command.arguments count] > 6 ? [[command.arguments objectAtIndex:6] intValue] : -1;
        int horAlign = [command.arguments count] > 7 ? [[command.arguments objectAtIndex:7] intValue] : -1;
        int verAlign = [command.arguments count] > 8 ? [[command.arguments objectAtIndex:8] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        if(horAlign >= 0)
            [LPAPI setItemHorizontalAlignment:horAlign];
        if(verAlign >= 0)
            [LPAPI setItemVerticalAlignment:verAlign];
        //
        [LPAPI drawQRCode:[[command arguments] objectAtIndex:0]
                        x:[[[command arguments] objectAtIndex:1] floatValue] 
                        y:[[[command arguments] objectAtIndex:2] floatValue]
                    width:width
                   height:height > 0 ? height : width
                 eccLevel:eccLevel >= 0 ? eccLevel : 0
           autoResetWidth:false
        ];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawImage:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:6])
    {
        int rotation = [command.arguments count] > 6 ? [[command.arguments objectAtIndex:6] intValue] : -1;
        int horAlign = [command.arguments count] > 7 ? [[command.arguments objectAtIndex:7] intValue] : -1;
        int verAlign = [command.arguments count] > 8 ? [[command.arguments objectAtIndex:8] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        if(horAlign >= 0)
            [LPAPI setItemHorizontalAlignment:horAlign];
        if(verAlign >= 0)
            [LPAPI setItemVerticalAlignment:verAlign];
        //
        [LPAPI drawImage:[[command arguments] objectAtIndex:0]
                       x:[[[command arguments] objectAtIndex:1] floatValue] 
                       y:[[[command arguments] objectAtIndex:2] floatValue]
                   width:[[[command arguments] objectAtIndex:3] floatValue]
                  height:[[[command arguments] objectAtIndex:4] floatValue]
               threshold:[[[command arguments] objectAtIndex:5] intValue]];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawFullImage:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:2])
    {
        int rotation = [command.arguments count] > 2 ? [[command.arguments objectAtIndex:2] intValue] : -1;
        int horAlign = [command.arguments count] > 3 ? [[command.arguments objectAtIndex:3] intValue] : -1;
        int verAlign = [command.arguments count] > 4 ? [[command.arguments objectAtIndex:4] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        if(horAlign >= 0)
            [LPAPI setItemHorizontalAlignment:horAlign];
        if(verAlign >= 0)
            [LPAPI setItemVerticalAlignment:verAlign];
        //
        [LPAPI drawImage:[[command arguments] objectAtIndex:0]
               threshold:[[[command arguments] objectAtIndex:1] intValue]];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawImageWithImage:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:6])
    {
        int rotation = [command.arguments count] > 6 ? [[command.arguments objectAtIndex:6] intValue] : -1;
        int horAlign = [command.arguments count] > 7 ? [[command.arguments objectAtIndex:7] intValue] : -1;
        int verAlign = [command.arguments count] > 8 ? [[command.arguments objectAtIndex:8] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        if(horAlign >= 0)
            [LPAPI setItemHorizontalAlignment:horAlign];
        if(verAlign >= 0)
            [LPAPI setItemVerticalAlignment:verAlign];
        //
        [LPAPI drawImageWithImage:[[command arguments] objectAtIndex:0]
                                x:[[[command arguments] objectAtIndex:1] floatValue] 
                                y:[[[command arguments] objectAtIndex:2] floatValue]
                            width:[[[command arguments] objectAtIndex:3] floatValue]
                           height:[[[command arguments] objectAtIndex:4] floatValue]
                        threshold:[[[command arguments] objectAtIndex:5] intValue]];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawFullImageWithImage:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:2])
    {
        int rotation = [command.arguments count] > 2 ? [[command.arguments objectAtIndex:2] intValue] : -1;
        int horAlign = [command.arguments count] > 3 ? [[command.arguments objectAtIndex:3] intValue] : -1;
        int verAlign = [command.arguments count] > 4 ? [[command.arguments objectAtIndex:4] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        if(horAlign >= 0)
            [LPAPI setItemHorizontalAlignment:horAlign];
        if(verAlign >= 0)
            [LPAPI setItemVerticalAlignment:verAlign];
        [LPAPI drawImageWithImage:[[command arguments] objectAtIndex:0]
                        threshold:[[[command arguments] objectAtIndex:1] intValue]];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawLine:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:4])
    {
        double x = [[command.arguments objectAtIndex:0] floatValue];
        double y = [[command.arguments objectAtIndex:1] floatValue];
        double width = [[command.arguments objectAtIndex:2] floatValue];
        double height = [[command.arguments objectAtIndex:3] floatValue];
        //
        NSArray<NSString *> *dashLens = [command.arguments count] > 4 ? [command argumentAtIndex:4] : nil;
        int rotation = [command.arguments count] > 5 ? [[command.arguments objectAtIndex:5] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        //
        if(dashLens != nil && dashLens.count > 0) {
            // 绘制虚线
            bool draw = true;
            int dashIndex = 0;
            double dashLen = 0;
            if(width > height) {
                // 循环绘制横线
                for(double offsetX = 0; offsetX < width;dashIndex++) {
                    if(dashIndex >= dashLens.count)
                        dashIndex = 0;
                    dashLen = [[dashLens objectAtIndex:dashIndex] floatValue];
                    //
                    if(draw) {
                        [LPAPI drawRectangleWithX:x + offsetX
                                                y:y
                                            width:dashLen
                                           height:height
                                        lineWidth:0.1
                                         isFilled:true];
                    }
                    //
                    offsetX += dashLen;
                    draw = !draw;
                }
            } else {
                // 循环绘制竖线int dashIndex = 0;
                for(double offsetY = 0; offsetY < height;dashIndex++) {
                    if(dashIndex >= dashLens.count)
                        dashIndex = 0;
                    dashLen = [[dashLens objectAtIndex:dashIndex] floatValue];
                    //
                    if(draw) {
                        [LPAPI drawRectangleWithX:x
                                                y:y + offsetY
                                            width:width
                                           height:dashLen
                                        lineWidth:0.1
                                         isFilled:true];
                    }
                    //
                    offsetY += dashLen;
                    draw = !draw;
                }
            }
        } else {
            // 绘制直线
            [LPAPI drawLineWithX:[[[command arguments] objectAtIndex:0] floatValue]
                               y:[[[command arguments] objectAtIndex:1] floatValue]
                           width:[[[command arguments] objectAtIndex:2] floatValue]
                          height:[[[command arguments] objectAtIndex:3] floatValue]];
        }
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawRectangle:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:6])
    {
        float x = [[command.arguments objectAtIndex:0] floatValue];
        float y = [[command.arguments objectAtIndex:1] floatValue];
        float width = [[command.arguments objectAtIndex:2] floatValue];
        float height = [[command.arguments objectAtIndex:3] floatValue];
        float lineWidth = [[command.arguments objectAtIndex:4] floatValue];
        float radius = [command.arguments count] > 5 ? [[command.arguments objectAtIndex:5] floatValue] : 0;
        bool fill = [command.arguments count] > 6 ? [[command.arguments objectAtIndex:6] boolValue] : false;
        int rotation = [command.arguments count] > 7 ? [[command.arguments objectAtIndex:7] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        //
        if(radius > 0) {
            [LPAPI drawRoundRectangleWithX:x
                                         y:y
                                     width:width
                                    height:height
                                 lineWidth:lineWidth
                                    radius:radius
                                  isFilled:fill];
        } else {
            [LPAPI drawRectangleWithX:x
                                    y:y
                                width:width
                               height:height
                            lineWidth:lineWidth
                             isFilled:fill];
        }
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawRoundRectangle:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:5])
    {
        float radius = [command.arguments count] > 5 ? [[command.arguments objectAtIndex:5] floatValue] : 0;
        bool fill = [command.arguments count] > 6 ? [[command.arguments objectAtIndex:6] boolValue] : false;
        int rotation = [command.arguments count] > 7 ? [[command.arguments objectAtIndex:7] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        //
        [LPAPI drawRoundRectangleWithX:[[[command arguments] objectAtIndex:0] floatValue]
                                     y:[[[command arguments] objectAtIndex:1] floatValue]
                                 width:[[[command arguments] objectAtIndex:2] floatValue]
                                height:[[[command arguments] objectAtIndex:3] floatValue]
                             lineWidth:[[[command arguments] objectAtIndex:4] floatValue]
                                radius:radius > 0 ? radius : 1.5
                              isFilled:fill];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawEllipse:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:6])
    {
        int rotation = [command.arguments count] > 6 ? [[command.arguments objectAtIndex:6] intValue] : -1;
        //
        if(rotation >= 0)
            [LPAPI setItemOrientation:rotation];
        //
        [LPAPI drawEllipseWithX:[[[command arguments] objectAtIndex:0] floatValue]
                              y:[[[command arguments] objectAtIndex:1] floatValue]
                          width:[[[command arguments] objectAtIndex:2] floatValue]
                         height:[[[command arguments] objectAtIndex:3] floatValue]
                      lineWidth:[[[command arguments] objectAtIndex:4] floatValue]
                       isFilled:[[[command arguments] objectAtIndex:5] boolValue]];
    }
    [self sendSuccessResult:command message:nil];
}

- (void)drawCircle:(CDVInvokedUrlCommand *)command
{
    if ([self isCommandTrue:command count:6])
    {
        [LPAPI drawCircleWithX:[[[command arguments] objectAtIndex:0] floatValue]
                             y:[[[command arguments] objectAtIndex:1] floatValue]
                         width:[[[command arguments] objectAtIndex:2] floatValue]
                        height:[[[command arguments] objectAtIndex:3] floatValue]
                     lineWidth:[[[command arguments] objectAtIndex:4] floatValue]
                      isFilled:[[[command arguments] objectAtIndex:5] boolValue]];
    }
    [self sendSuccessResult:command message:nil];
}

@end
