
/**
 * Created by HuDianxing on 2020-09-01.
 *
 * 德佟电子标签打印机 JavaScript 接口的封装。
 *
 * Copyright(C) 2011~2020, 上海道臻信息技术有限公司
 *
 */
 
    var exec = require("cordova/exec");
    var api = {};
    var SERVICE_ANDROID = "LPAPIPlugin";
    var SERVICE_IOS = "ocLPAPIModel";
    var deviceMap = new Map();

    /**
     * @typedef IExecOptions
     * @property {(res: any)=> void} success 执行成功时的回调函数；
     * @property {(res: any) => void} fail 执行失败时的回调函数；
     * @property {(res: any) => void} complete 执行成功或者失败时的回调函数；
     */

    /**用于判断当前js运行的环境是什么平台 */
    var execUtils = {
        isAndroid: function () {
            return navigator.userAgent.match(/Android/i);
        },
        isIOS: function () {
            return navigator.userAgent.match(/iPhone|iPad|iPod/i);
        },
        /**
         *
         * @param {string} action 用于关联底层对应的方法名称；
         * @param {IExecOptions} options 入参及回调函数；
         * @returns void
         */
        execAndroid: function (action, options) {
            console.log(`#### 【Request】: [${SERVICE_ANDROID}.${action}]`);
            // 返回值规则：
            // 如果失败，直接返回false;
            // 如果成功，直接返回成功的详细信息；
            const opts = options || {};
            const datas = Object.keys(opts)
                .filter((v) => typeof opts[v] !== "function" && opts[v] !== undefined && opts[v] !== null)
                .reduce((prev, key) => {
                    prev[key] = opts[key];
                    return prev;
                }, {});
            console.log(datas);
            return new Promise((resolve) => {
                exec(
                    (v) => {
                        // 成功回调：onSuccess();
                        console.log(`#### 【Response.success】- [${SERVICE_ANDROID}.${action}]:`);
                        console.log(v);
                        opts.success && opts.success(v);
                        opts.complete && opts.complete(v);
                        resolve(v);
                    },
                    (v) => {
                        // 错误回调: onError();
                        console.log(`#### 【Response.fail】- [${SERVICE_ANDROID}.${action}]:`);
                        console.log(v);
                        opts.fail && opts.fail(v);
                        opts.complete && opts.complete(v);
                        resolve(null);
                    },
                    SERVICE_ANDROID,
                    action,
                    datas
                );
            });
        },
        /**
         *
         * @param {string} action 要执行的底层函数名称；
         * @param {any[]} datas 参数列表；
         * @param {IExecOptions} options 回调函数；
         * @returns void
         */
        execIOS: function (action, datas, options) {
            console.log(`#### 【Request】: [${SERVICE_IOS}.${action}]`);
            console.log(datas);
            const opts = options || {};
            //
            return new Promise((resolve) => {
                exec(
                    (v) => {
                        // 成功回调：onSuccess();
                        // console.log(`#### 【Response.success】- [${SERVICE_IOS}.${action}]:`);
                        opts.success && opts.success(v);
                        opts.complete && opts.complete(v);
                        resolve(v);
                    },
                    (v) => {
                        // 错误回调: onError();
                        // console.log(`#### 【Response.fail】- [${SERVICE_IOS}.${action}]:`);
                        opts.fail && opts.fail(v);
                        opts.complete && opts.complete(v);
                        resolve(false);
                    },
                    SERVICE_IOS,
                    action,
                    datas || []
                );
            });
        },
        /**
         * @param {IExecOptions} options
         * @param {any} value
         */
        processSuccessResult(options, value) {
            const opts = options || {};
            opts.success && opts.success(value);
            opts.complete && opts.complete(value);
            return Promise.resolve(value);
        },
        /**
         *
         * @param {IExecOptions} options
         * @param {any} value
         */
        processFailResult(options, value) {
            const opts = options || {};
            opts.fail && opts.fail(value);
            opts.complete && opts.complete(value);
            return Promise.resolve(value);
        },
        /**
         * 将顺序传参转换为对象模式；
         * @param {string} keys 对象参数名称列表；
         * @param {any} values 对象参数内容列表；
         * @returns {any}
         */
        processExecData(keys, values) {
            keys = typeof keys === "string" ? [keys] : keys || [];
            values = values || [];
            //
            if (typeof values[0] === "object") {
                return values[0];
            }
            //
            var obj = {};
            for (var i = 0; i < keys.length; i++) {
                if (values[i] !== undefined && values[i] !== null) {
                    obj[keys[i]] = values[i];
                }
            }
            return obj;
        },
        base64ToArrayBuffer(data) {
            // 删除所有的空白字符
            data = data.replace(/\s/g, "");
            var base64 = data.replace(/\-/g, "+").replace(/_/g, "/");
            try {
                var rawData = atob(base64);
                var output = new Uint8Array(rawData.length);
                for (let i = 0; i < rawData.length; i++) {
                    output[i] = rawData.charCodeAt(i);
                }
                return output.buffer;
            } catch (error) {
                return undefined;
            }
        },
        arrayBufferToBase64(buffer) {
            if (!buffer) return "";
            //
            var data = new Uint8Array(buffer);
            try {
                return btoa(new TextDecoder().decode(data));
            } catch (error) {
                return btoa(data.reduce((prev, byte) => prev + String.fromCharCode(byte), ""));
            }
        }
    };

    /***************************************************************************
     * 打印机的打开与断开操作。
     **************************************************************************/

    /**
     * 执行相关初始化工作，在Android环境下会动态申请蓝牙操作权限。
     *
     * @param {Object} options
     *            接口初始化相关参数。
     * @param {string} options.models
     *            需要兼容的打印机型号。
     * @param {number} options.clientType
     *            打印机客户类型。
     * @param {boolean} options.autoDisconnect
     *            息屏的时候是否自动关闭打印机，默认为true。
     */
    api.init = function (options) {
        options = execUtils.processExecData(["models"], arguments);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("initPlugin", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS(
                "initPlugin",
                [options.models || "", typeof options.clientType === "number" ? options.clientType : -1],
                options
            );
        } else {
            return Promise.resolve(false);
        }
    };

    /**
     * 在Android中动态申请蓝牙权限。
     */
    api.requestBluetoothAuth = function (options) {
        const opts = typeof options === "function" ? { complete: options } : options;
        if (execUtils.isAndroid()) {
            // return LPAPIPlugin.requestBluetoothAuth(opts);
            return execUtils.execAndroid("requestBluetoothAuth", opts);
        } else {
            return execUtils.processSuccessResult(opts, true);
        }
    };

    /**
     * 设置要搜索的蓝牙设备型号，默认搜索所有设备。
     * @param {string} models 设置要搜索的蓝牙设备型号。多个型号间用英文符号";"进行分割。
     */
    api.setPrinterModels = function (models) {
        const options = execUtils.processExecData(["models", "complete"], arguments);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("setPrinterModels", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("setSupportPrefixs", [options.models || ""], options);
        } else {
            return execUtils.processFailResult(false);
        }
    };
    /**
     * 开始扫描蓝牙设备。
     *
     * @param {Object} options
     *            设备扫描配置信息；
     * @param {number|undefined} timeout
     *            设备搜索超时时间；
     * @param {string|undefined} models
     *            如果有值，则只搜索目标型号；
     * @param {(devices: any) => void} onDeviceFound
     *            搜索到设备时的回调函数；
     * @param {(status: number) => void} onStatusChanged
     *            搜索状态发生变化时的回调函数；
     */
    api.startScan = function (options) {
        options = execUtils.processExecData(["models"], arguments);
        deviceMap.clear();
        if (execUtils.isAndroid()) {
            execUtils.execAndroid("startBleDiscovery", {
                models: options.models,
                timeout: options.timeout,
                interval: options.interval,
                /**
                 *
                 * @param {Object} res
                 * @param {number} res.statusCode
                 * @param {any} res.resultInfo
                 */
                success: (res) => {
                    if (res.statusCode === 0) {
                        // 启动成功
                        options.onStatusChanged && options.onStatusChanged(true);
                    } else if (res.statusCode === 1) {
                        // 检测到设备
                        const deviceList = res.resultInfo || [];
                        for (const item of deviceList) {
                            if (item.deviceId) {
                                deviceMap.set(item.deviceId, item);
                            }
                        }
                        options.onDeviceFound && options.onDeviceFound(Array.from(deviceMap.values()));
                        // 如果超时时间为0，则停止搜索
                        if (typeof options.timeout === "number" && options.timeout <= 0) {
                            api.stopScan();
                        }
                    } else {
                        if (res.statusCode === 2) {
                            // 搜索结束
                        }
                        // 蓝牙权限申请失败，或者其他错误信息。
                        scanning = false;
                        options.onStatusChanged && options.onStatusChanged(false);
                    }
                },
                fail: () => {
                    // 启动异常，搜索结束
                    options.onStatusChanged && options.onStatusChanged(false);
                },
            });
        } else if (execUtils.isIOS()) {
            options.onStatusChanged && options.onStatusChanged(true);
            execUtils.execIOS("scanPrinters", [options.models || options.name || ""], {
                /**
                 *
                 * @param {string[]} devices
                 */
                success: (devices) => {
                    const list = (devices || []).map((name) => {
                        return typeof name === "string" ? { name: name } : name;
                    });
                    options.onDeviceFound && options.onDeviceFound(list);
                    options.onStatusChanged && options.onStatusChanged(false);
                },
                fail: () => {
                    options.onStatusChanged && options.onStatusChanged(false);
                },
            });
        } else {
            execUtils.processFailResult(undefined);
        }
    };

    api.stopScan = function (options) {
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("stopBleDiscovery", options);
        } else {
            return processSuccessResult(options);
        }
    };
    /**
     * 以字符串的形式返回已经安装过的所有打印机名称，不同打印机名称间以英文","分隔。
     * @param names 打印机型号，
     *      空字符串表示返回所有型号的打印机
     *      要获取多个型号的打印机，多个型号之间可以用";"进行分割；
     * @param success 成功回调函数；
     * @param fail 失败回调函数；
     * 平台支持：Android+IOS；
     */
    api.getPrinters = function (names) {
        if (execUtils.isAndroid()) {
            const opts = execUtils.processExecData("name", arguments);
            return execUtils.execAndroid("getAllPrinters", opts);
        } else if (execUtils.isIOS()) {
            // 设置型号限制
            names = names || "";
            if (names) {
                execUtils.execIOS("setSupportPrefixs", names.split(";"));
            }
            return execUtils.execIOS("scanPrinters", names.split(";")).then((list) => {
                if (list) {
                    return list.map((v) => (typeof v === "string" ? { name: v } : v));
                } else {
                    return [];
                }
            });
        } else {
            return execUtils.processFailResult(false);
        }
    };

    /**
     * 打开指定名称的打印机(异步调用)。
     * @param {Object} options 打印机链接相关参数信息。
     * @param printerName
     *            打印机名称。打印机名称类型：
     *            1、空字符串：打开当前客户端系统上的第一个支持的打印机；
     *            2、打印机型号：例如：DT20S；
     *            3、打印机名称：例如：DT20S-60901687;
     *            4、MAC地址：打开指定地址的打印机，例如：00:18:E4:0C:68:CA。
     * @param {string} options.name
     *            要连接的目标设备名称。
     * @param {string} options.deviceId
     *            要连接的目标设备ID。
     * @param {number} options.clientType
     *            打印机客户类型。
     * @param {boolean} options.bleMode
     *            是否使用ble模式来链接目标打印机？
     * @param success 成功回调函数；
     * @param fail 失败回调函数；
     * 平台支持：Android+IOS；
     */
    api.openPrinter = function (options) {
        options = execUtils.processExecData(["name", "complete"], arguments);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("openPrinter", options);
        } else if (execUtils.isIOS()) {
            // return lpapiModel.jsOpenPrinter([printerName]);
            return execUtils.execIOS(
                "openPrinter",
                [
                    options.name || options.printerName,
                    typeof options.clientType === "number" && options.clientType >= 0 ? options.clientType : -1,
                ],
                {
                    complete: options.callback || options.complete,
                }
            );
        }
    };

    /**
     * 断开当前打印机的连接。
     * 平台支持：Android+IOS；
     */
    api.closePrinter = function () {
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("closePrinter");
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("closePrinter");
        }
    };

    /**
     * 直接向打印机发送指令。
     * 平台支持：Android；
     */
    api.writeCommands = function (options) {
        var data = options.data || options.value;
        if(data instanceof Uint8Array || ArrayBuffer.isView(data)) {
            data = execUtils.arrayBufferToBase64(data);
        }
        if (execUtils.isAndroid()) {
            execUtils.execAndroid("writeCommands", {
                data: options.models,
                complete: (res) => {
                    if (res.statusCode === 0 || res.statusCode === 3) {
                        // 数据发送成功
                        var result = res.resultInfo;
                        if(typeof result === "string") {
                            result = execUtils.base64ToArrayBuffer(result);
                        }
                        if(result) {
                            execUtils.processSuccessResult(options, result);
                        } else {
                            execUtils.processFailResult(options, res);
                        }
                    } else {
                        execUtils.processFailResult(options, res);
                    }
                }
            });
        } else if (execUtils.isIOS()) {
            execUtils.processFailResult(options, false);
        }
    };

    /**
     * 得到当前使用的打印机名称。
     *
     * @return 如果已连接打印机，则返回当前使用的打印机名称，否则返回空字符串。
     * 平台支持：Android+IOS；
     */
    api.getPrinterName = function () {
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("getPrinterName");
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("connectingPrinterName");
        }
    };

    /**
     * 获取当前已连接打印机的详细信息；
     *
     * @param success 结果回调函数；
     * 平台支持：Android；
     */
    api.getPrinterInfo = function () {
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("getPrinterInfo");
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("getPrinterInfo");
        }
    };

    /**
     * 判断当前打印机是否打开（连接成功）？
     * 平台支持：Android；
     */
    api.isPrinterOpened = function () {
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("isPrinterOpened");
        } else {
            return execUtils.processFailResult(false);
        }
    };

    /***************************************************************************
     * 打印任务的开始，分页，结束等操作。
     **************************************************************************/

    /**
     * 以指定的参数，开始一个打印任务。
     *
     * @param {Object} options 打印任务参数信息;
     * @param {number} options.width 标签宽度（单位mm）。
     * @param {number} options.height 标签高度（单位mm）。
     * @param {number} options.orientation 标签打印方向。0：不旋转；90：顺时针旋转90度；180：旋转180度；270：逆时针旋转90度。
     * @param {string} options.jobName 打印任务名称；
     * @param {string} options.imageUrl 打印任务预览图片URL；
     * @param {string} options.backgroundImage BASE64 格式的打印任务预览图片；
     * @return 成功与否？
     * 平台支持：Android+IOS；
     */
    api.startJob = function (options) {
        options = execUtils.processExecData(["width", "height", "orientation", "jobName", "background"], arguments);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("startJob", options);
        } else if (execUtils.isIOS()) {
            // return lpapiModel.jsStartPrint([width, height, orientation, 1]);
            return execUtils.execIOS("startPrint", [
                options.width || 0,
                options.height || 0,
                options.orientation || 0,
                options.jobName || "",
                typeof options.background === "number" && options.background >= 0 ? options.background : -1,
            ]);
        }
    };

    /**
     * 开始一个打印页面。
     * @param success 成功回调函数；
     * @param fail 失败回调函数；
     * 平台支持：Android；
     */
    api.startPage = function () {
        if (execUtils.isAndroid()) {
            // return LPAPIPlugin.startPage();
            return execUtils.execAndroid("startPage");
        } else {
            execUtils.processSuccessResult(true);
        }
    };

    /**
     * 结束一个打印页面。
     * 平台支持：Android；
     */
    api.endPage = function () {
        if (execUtils.isAndroid()) {
            // return LPAPIPlugin.endPage();
            return execUtils.execAndroid("endPage");
        } else {
            return execUtils.processSuccessResult(true);
        }
    };

    /**
     * 结束绘图任务。
     * 调用endJob()后，并未将打印任务提交给打印机，但是可以通过调用getJobPages(),然后以图片形式获取打印任务；
     * 平台支持：Android；
     */
    api.endJob = function () {
        if (execUtils.isAndroid()) {
            // return LPAPIPlugin.endJob();
            return execUtils.execAndroid("endJob");
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("endPrint");
        }
    };

    /**
     * 提交打印数据，进行真正的打印。
     * @param {Object} options 
     *            设置相关打印参数。
     * @param {number} options.gapType 
     *            打印任务纸张类型。
     * @param {number} options.printDarkness 
     *            打印任务的打印浓度。
     * @param {number} options.printSpeed
     *            打印速度。
     * @param success 成功回调函数；
     * @param fail 失败回调函数；
     * 平台支持：Android+IOS；
     */
    api.commitJob = function (options) {
        if (execUtils.isAndroid()) {
            // return LPAPIPlugin.commitJob();
            return execUtils.execAndroid("commitJob", options);
        } else if (execUtils.isIOS()) {
            // return lpapiModel.jsCommitPrint();
            return execUtils.execIOS("commitPrint", [
                typeof options.gapType === "number" ? options.gapType : -1,
                options.gapLength || 0,
                typeof options.printDarkness === "number" ? options.printDarkness : -1,
                typeof options.printSpeed === "number" ? options.printSpeed : -1,
                options.printCopies || 0,
            ], options);
        }
    };

    /**
     * 调用函数endJob() 之后可以调用该函数，获取base64图片格式的打印任务；
     * @param success 数据获取成功回调函数(返回值);
     * 平台支持：Android；
     */
    api.getJobPages = function (success) {
        const opts = typeof success === "function" ? { complete: success } : success || {};
        if (execUtils.isAndroid()) {
            // return LPAPIPlugin.getJobPages(success);
            return execUtils.execAndroid("getJobPages", opts);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("previewImage", [], opts);
        }
    };

    /**
     * 打印base64格式的图片对象
     * @param {Object} options
     *            图片打印相关参数。
     * @param {string} options.data
     *            base64格式图片或者URL路径；
     * @param {string} options.image
     *            base64格式图片或者URL路径；
     * @param args 配置其他打印参数；
     * 平台支持：Android；
     */
    api.printImage = function (options) {
        options = execUtils.processExecData(["data"], arguments);
        if (execUtils.isAndroid()) {
            // return LPAPIPlugin.printImage();
            return execUtils.execAndroid("printImage", options);
        } else if (execUtils.isIOS()) {
            return execUtils.processFailResult(false);
        }
    };

    /***************************************************************************
     * 打印参数设置。
     **************************************************************************/

    /**
     * 设置后续打印内容的字体名称。
     *
     * @param {string} fontName
     *            后续绘制内容的字体名称。
     */
    api.setFontName = function (fontName) {
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("setFontName", {
                fontName: fontName || "",
            });
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("setFontName", [fontName || ""]);
        }
    };
    /**
     * 设置相关绘制参数。
     *
     * @param {Object} options 设置相关绘制参数。
     * @returns {Promise<any>}
     */
    api.setDrawParam = function (options) {
        options = execUtils.processExecData(["name", "value"], arguments);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("setDrawParam", options);
        } else if (execUtils.isIOS()) {
            // return execUtils.execIOS("setFontName", [
            //     options.name || "",
            //     options.value || "",
            // ]);
            return Promise.resolve(false);
        }
    };
    /**
     * 设置打印动作的旋转角度。
     *
     * @param orientation
     *            orientation: 旋转角度。参数描述如下：
     *            0：不旋转；
     *            90：顺时针旋转90度；
     *            180：旋转180度；
     *            270：逆时针旋转90度。
     * 平台支持：Android + IOS;
     */
    api.setItemOrientation = function (orientation) {
        const opts = execUtils.processExecData("orientation", [orientation]);
        if (execUtils.isAndroid()) {
            // return LPAPIPlugin.setItemOrientation({
            // 	orientation: orientation || 0
            // });
            return execUtils.execAndroid("setItemOrientation", opts);
        } else if (execUtils.isIOS()) {
            // return lpapiModel.jsSetItemOrientation([orientation]);
            return execUtils.execIOS("setItemOrientation", [opts.orientation || 0]);
        }
    };

    /**
     * 设置打印动作的水平对齐方式。
     *
     * @param alignment
     *            水平对齐方式。参数描述如下：
     *            0：水平居左（默认方式）；
     *            1：水平居中；
     *            2：水平居右。
     * 平台支持：Android + IOS；
     */
    api.setItemHorizontalAlignment = function (alignment) {
        const opts = execUtils.processExecData("alignment", [alignment]);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("setItemHorizontalAlignment", opts);
        } else if (execUtils.isIOS()) {
            // return lpapiModel.jsSetItemHorizontalAlignment([alignment]);
            return execUtils.execIOS("setItemHorizontalAlignment", [opts.alignment || 0]);
        }
    };

    /**
     * 设置打印动作的垂直对齐方式。
     *
     * @param alignment
     *            垂直对齐方式，参数描述如下：
     *            0：垂直居上（默认方式）；
     *            1：垂直居中；
     *            2：垂直居下。
     * 平台支持：Android + IOS；
     */
    api.setItemVerticalAlignment = function (alignment) {
        const opts = execUtils.processExecData("alignment", [alignment]);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("setItemVerticalAlignment", opts);
        } else if (execUtils.isIOS()) {
            // return lpapiModel.jsSetItemVerticalAlignment([alignment]);
            return execUtils.execIOS("setItemVerticalAlignment", [opts.alignment || 0]);
        }
    };

    /***************************************************************************
     * 打印对象的绘制操作。
     **************************************************************************/

    /**
     * 打印文本。
     *
     * @param {Object} options
     *            字符串绘制相关参数；
     * @param {string} options.text
     *            文本内容。
     * @param {number} options.x
     *            打印对象的位置(单位mm)。
     * @param {number} options.y
     *            打印对象的位置(单位mm)。
     * @param {number} options.width
     *            打印对象的宽度(单位mm)。
     * @param {number} options.height
     *            打印对象的高度(单位mm)。
     *            height 为 0 时，真正的打印文本高度会根据内容来扩展；否则当指定的高度不足以打印指定的文本时，会自动缩小字体来适应指定的高度进行文本打印。
     * @param {number} options.fontHeight
     *            文本的字体高度(单位mm)。
     * @param {number} options.fontStyle
     *            文本的字体风格（可按位组合），可以不指定，默认为0（正常）。0：正常；1：粗体；2：斜体；3：粗斜体 ；4：下划线；8：删除线。
     * @param {number} options.orientation
     *            绘制内容的旋转方向。
     * @param {number} options.horizontalAlignment
     *            绘制内容的水平对齐方式。
     * @param {number} options.verticalAlignment
     *            绘制内容的垂直对齐方式。
     * @return 打印成功与否？
     */
    api.drawText = function (options) {
        options = execUtils.processExecData(
            ["text", "x", "y", "width", "height", "fontHeight", "fontStyle", "fontName"],
            arguments
        );
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawText", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawText", [
                options.text || "",
                options.x || 0,
                options.y || 0,
                options.width || 0,
                options.height || 0,
                options.fontName || "",
                options.fontHeight || 0,
                options.fontStyle || 0,
                options.charSpace || 0,
                options.lineSpace || 0,
                typeof options.orientation === "number" ? options.orientation : -1,
                typeof options.horizontalAlignment === "number" ? options.horizontalAlignment : -1,
                typeof options.verticalAlignment === "number" ? options.verticalAlignment : -1,
            ]);
        }
    };

    /**
     * 打印一维条码。
     *
     * @param {Object} options
     *            一维码绘制相关参数；
     * @param {string} options.text
     *            需要绘制的一维条码的内容。
     * @param {number} options.x
     *            绘制的一维条码的左上角水平位置（单位mm）。
     * @param {number} options.y
     *            绘制的一维条码的左上角垂直位置（单位mm）。
     * @param {number} options.width
     *            一维条码的整体显示宽度。
     * @param {number} options.height
     *            一维条码的显示高度（包括供人识读文本）。
     * @param {number} options.textHeight
     *            供人识读文本的高度（单位mm），建议为3毫米。
     * @param {number} options.type
     *            一维条码的编码类型参考文档。
     * @param {number} options.orientation
     *            绘制内容的旋转方向。
     * @param {number} options.horizontalAlignment
     *            绘制内容的水平对齐方式。
     * @param {number} options.verticalAlignment
     *            绘制内容的垂直对齐方式。
     * @return 打印成功与否？
     */
    api.drawBarcode = function (options) {
        options = execUtils.processExecData(["text", "x", "y", "width", "height", "textHeight", "type"], arguments);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("draw1DBarcode", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawBarcode", [
                options.text || "",
                options.x || 0,
                options.y || 0,
                options.width || 0,
                options.height || 0,
                options.fontName || "",
                options.textHeight || 0,
                options.fontStyle || 0,
                typeof options.textFlag === "number" ? options.textFlag : 2,
                options.type || 0,
                typeof options.orientation === "number" ? options.orientation : -1,
                typeof options.horizontalAlignment === "number" ? options.horizontalAlignment : -1,
                typeof options.verticalAlignment === "number" ? options.verticalAlignment : -1,
            ]);
        }
    };

    /**
     * 打印 QRCode 二维码。
     *
     * @param {Object} options
     *            二维码绘制参数；
     * @param {string} options.text
     *            需要绘制的QRCode二维码的内容。
     * @param {number} options.x
     *            绘制的QRCode二维码的左上角水平位置（单位mm）。
     * @param {number} options.y
     *            绘制的QRCode二维码的左上角垂直位置（单位mm）。
     * @param {number} options.width
     *            绘制的QRCode二维码的水平宽度（单位mm）。
     * @param {number} options.height
     *            绘制的QRCode二维码的水平高度（单位mm）。
     * @param {number} options.eccLevel
     *            绘制的QRCode二维码的纠错级别；
     * @param {number} options.orientation
     *            绘制内容的旋转方向。
     * @param {number} options.horizontalAlignment
     *            绘制内容的水平对齐方式。
     * @param {number} options.verticalAlignment
     *            绘制内容的垂直对齐方式。
     * @return 打印成功与否？
     */
    api.drawQRCode = function (options) {
        options = execUtils.processExecData(["text", "x", "y", "width", "height", "eccLevel"], arguments);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("draw2DQRCode", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawQRCode", [
                options.text || "",
                options.x || 0,
                options.y || 0,
                options.width || 0,
                options.height || 0,
                options.eccLevel || 0,
                typeof options.orientation === "number" ? options.orientation : -1,
                typeof options.horizontalAlignment === "number" ? options.horizontalAlignment : -1,
                typeof options.verticalAlignment === "number" ? options.verticalAlignment : -1,
            ]);
        }
    };

    /**
     * 绘制图片内容。
     *
     * @param {Object} options
     *            图片绘制参数；
     * @param {string} options.image
     *            图片路径或者BASE64字符串。
     * @param {string} options.data
     *            图片路径或者BASE64字符串, 同options.image。
     * @param {number} options.x
     *            打印对象在水平方向上的位置(单位mm)。
     * @param {number} options.y
     *            打印对象在垂直方向上的位置(单位mm)。
     * @param {number} options.width
     *            打印对象的宽度(单位mm)。
     * @param {number} options.height
     *            打印对象的高度(单位mm)。
     * @param {number} options.threshold
     *            绘制位图的灰度阀值。
     *            256 表示绘制灰度图片；
     *            257 表示绘制原色图片；
     *            0～255表示绘制黑白图片，原图颜色>=灰度阀值的点会被认为是白色，而原图颜色<灰度阀值的点会被认为是黑色。默认值为192。
     * @param {number} options.orientation
     *            绘制内容的旋转方向。
     * @param {number} options.horizontalAlignment
     *            绘制内容的水平对齐方式。
     * @param {number} options.verticalAlignment
     *            绘制内容的垂直对齐方式。
     * @return 打印成功与否？
     */
    api.drawImage = function (options) {
        options = execUtils.processExecData(["data", "x", "y", "width", "height", "threshold"], arguments);
        console.log(options);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawImage", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawImage", [
                options.data || options.image || "",
                options.x || 0,
                options.y || 0,
                options.width || 0,
                options.height || 0,
                options.threshold || 0,
                typeof options.orientation === "number" ? options.orientation : -1,
                typeof options.horizontalAlignment === "number" ? options.horizontalAlignment : -1,
                typeof options.verticalAlignment === "number" ? options.verticalAlignment : -1,
            ]);
        }
    };

    /**
     * 以指定的线宽，打印矩形框。
     *
     * @param {Object} options 绘制参数；
     * @param {number} options.x
     *            绘制的矩形框的左上角水平位置（单位mm）。
     * @param {number} options.y
     *            绘制的矩形框的左上角垂直位置（单位mm）。
     * @param {number} options.width
     *            绘制的矩形框的水平宽度（单位mm）。
     * @param {number} options.height
     *            绘制的矩形框的垂直高度（单位mm）。
     * @param {number} options.cornerWidth
     *            绘制的矩形的圆角半径，默认为0。
     * @param {number} options.lineWidth
     *            矩形框的线宽（单位mm）。矩形框的线宽是向矩形框内部延伸的。
     * @param {boolean} options.fill
     *            是否绘制填充矩形？
     * @param {number} options.orientation
     *            绘制内容的旋转方向。
     * @return 打印成功与否？
     */
    api.drawRectangle = function (options) {
        options = execUtils.processExecData(["x", "y", "width", "height", "lineWidth", "fill"], arguments);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawRectangle", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawRectangle", [
                options.x || 0,
                options.y || 0,
                options.width || 0,
                options.height || 0,
                options.lineWidth || 0,
                options.cornerWidth || 0,
                options.fill || false,
                typeof options.orientation === "number" ? options.orientation : -1,
            ]);
        }
    };

    /**
     * 打印填充的矩形框。
     *
     * @param {Object} options 绘制参数
     * @param {number} options.x
     *            绘制的填充矩形框的左上角水平位置（单位mm）。
     * @param {number} options.y
     *            绘制的填充矩形框的左上角垂直位置（单位mm）。
     * @param {number} options.width
     *            绘制的填充矩形框的水平宽度（单位mm）。
     * @param {number} options.height
     *            绘制的填充矩形框的垂直高度（单位mm）。
     * @param {number} options.cornerWidth
     *            绘制的填充矩形的圆角半径，默认为0。
     * @param {number} options.orientation
     *            绘制内容的旋转方向。
     * @return 打印成功与否？
     */
    api.fillRectangle = function (options) {
        options = execUtils.processExecData(["x", "y", "width", "height", "cornerWidth"], arguments);
        options.fill = true;
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawRectangle", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawRectangle", [
                options.x || 0,
                options.y || 0,
                options.width || 0,
                options.height || 0,
                0.1,
                options.cornerWidth || 0,
                options.fill,
                typeof options.orientation === "number" ? options.orientation : -1,
            ]);
        }
    };

    /**
     * 以指定的线宽，打印圆角矩形框。
     *
     * @param {Object} options
     *            圆角矩形绘制相关参数；
     * @param {number} options.x
     *            绘制的圆角矩形框的左上角水平位置（单位mm）。
     * @param {number} options.y
     *            绘制的圆角矩形框的左上角垂直位置（单位mm）。
     * @param {number} options.width
     *            绘制的圆角矩形框的水平宽度（单位mm）。
     * @param {number} options.height
     *            绘制的圆角矩形框的垂直高度（单位mm）。
     * @param {number} options.cornerWidth
     *            圆角宽度（单位mm）。
     * @param {number} options.cornerHeight
     *            圆角高度（单位mm）。
     * @param {number} options.lineWidth
     *            圆角矩形框的线宽（单位mm）。圆角矩形框的线宽是向圆角矩形框内部延伸的。
     * @param {number} options.fill
     *            是否绘制填充圆角矩形？
     * @param {number} options.orientation
     *            绘制内容的旋转方向。
     * @return 打印成功与否？
     */
    api.drawRoundRectangle = function (options) {
        options = execUtils.processExecData(
            ["x", "y", "width", "height", "cornerWidth", "cornerHeight", "lineWidth", "fill"],
            arguments
        );
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawRoundRectangle", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawRoundRectangle", [
                options.x || 0,
                options.y || 0,
                options.width || 0,
                options.height || 0,
                options.lineWidth || 0,
                options.cornerWidth || 0,
                options.fill || false,
                typeof options.orientation === "number" ? options.orientation : -1,
            ]);
        }
    };

    /**
     * 打印填充的圆角矩形框。
     *
     * @param {Object} options
     *            填充圆角矩形绘制相关参数；
     * @param {number} options.x
     *            绘制的填充圆角矩形框的左上角水平位置（单位mm）。
     * @param {number} options.y
     *            绘制的填充圆角矩形框的左上角垂直位置（单位mm）。
     * @param {number} options.width
     *            绘制的填充圆角矩形框的水平宽度（单位mm）。
     * @param {number} options.height
     *            绘制的填充圆角矩形框的垂直高度（单位mm）。
     * @param {number} options.cornerWidth
     *            圆角宽度（单位mm）。
     * @param {number} options.cornerHeight
     *            圆角高度（单位mm）。
     * @param {number} options.orientation
     *            绘制内容的旋转方向。
     * @return 打印成功与否？
     */
    api.fillRoundRectangle = function (options) {
        options = execUtils.processExecData(["x", "y", "width", "height", "cornerWidth", "cornerHeight"], arguments);
        options.fill = true;
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawRoundRectangle", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawRoundRectangle", [
                options.x || 0,
                options.y || 0,
                options.width || 0,
                options.height || 0,
                0.1,
                options.cornerWidth || 0,
                options.fill,
                typeof options.orientation === "number" ? options.orientation : -1,
            ]);
        }
    };

    /**
     * 以指定的线宽，打印椭圆/圆。
     *
     * @param {Object} options
     *            椭圆绘制相关参数；
     * @param {number} options.x
     *            绘制的椭圆的左上角水平位置（单位mm）。
     * @param {number} options.y
     *            绘制的椭圆的左上角垂直位置（单位mm）。
     * @param {number} options.width
     *            绘制的椭圆的水平宽度（单位mm）。
     * @param {number} options.height
     *            绘制的椭圆的垂直高度（单位mm）。
     * @param {number} options.lineWidth
     *            椭圆的线宽（单位mm）。椭圆的线宽是向椭圆内部延伸的。
     * @param {boolean} options.fill
     *            是否需要绘制填充椭圆？
     * @param {number} options.orientation
     *            绘制内容的旋转方向。
     * @return 打印成功与否？
     */
    api.drawEllipse = function (options) {
        options = execUtils.processExecData(["x", "y", "width", "height", "lineWidth", "fill"], arguments);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawEllipse", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawEllipse", [
                options.x || 0,
                options.y || 0,
                options.width || 0,
                options.height || 0,
                options.lineWidth || 0,
                options.fill || false,
                typeof options.orientation === "number" ? options.orientation : -1,
            ]);
        }
    };

    /**
     * 打印填充的椭圆/圆。
     *
     * @param {Object} options
     *            椭圆绘制相关参数；
     * @param {number} options.x
     *            绘制的填充椭圆的左上角水平位置（单位mm）。
     * @param {number} options.y
     *            绘制的填充椭圆的左上角垂直位置（单位mm）。
     * @param {number} options.width
     *            绘制的填充椭圆的水平宽度（单位mm）。
     * @param {number} options.height
     *            绘制的填充椭圆的垂直高度（单位mm）。
     * @param {number} options.orientation
     *            绘制内容的旋转方向。
     * @return 打印成功与否？
     */
    api.fillEllipse = function (options) {
        options = execUtils.processExecData(["x", "y", "width", "height", "fill"], arguments);
        options.fill = true;
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawEllipse", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawEllipse", [
                options.x || 0,
                options.y || 0,
                options.width || 0,
                options.height || 0,
                0.1,
                options.fill || 0,
                typeof options.orientation === "number" ? options.orientation : -1,
            ]);
        }
    };

    /**
     * 以指定的线宽，打印圆。
     *
     * @param {Object} options
     *            正圆绘制相关参数；
     * @param {number} options.x
     *            绘制的圆形的X轴圆心位置（单位mm）。
     * @param {number} options.y
     *            绘制的圆形的Y轴圆心位置（单位mm）。
     * @param {number} options.radius
     *            绘制的圆的半径（单位mm）。
     * @param {number} options.width
     *            正圆所在区域的宽度。
     * @param {number} options.height
     *            正圆所在区域的高度。
     * @param {number} options.lineWidth
     *            圆的线宽（单位mm）。圆的线宽是向圆内部延伸的。
     * @param {boolean} options.fill
     *            是否需要绘制填充正圆？
     * @return 打印成功与否？
     */
    api.drawCircle = function (options) {
        options = execUtils.processExecData(["x", "y", "radius", "lineWidth", "fill"], arguments);
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawCircle", options);
        } else if (execUtils.isIOS()) {
            const x = options.x || 0;
            const y = options.y || 0;
            return execUtils.execIOS("drawCircle", [
                !options.radius ? x : x - options.radius,
                !options.radius ? y : y - options.radius,
                options.radius ? options.radius * 2 : options.width || 0,
                options.radius ? options.radius * 2 : options.height || options.width || 0,
                options.lineWidth || 0,
                options.fill || false,
            ]);
        }
    };

    /**
     * 打印填充的圆。
     *
     * @param {Object} options 绘制参数；
     * @param {number} options.x
     *            绘制的圆形的X轴圆心位置（单位mm）。
     * @param {number} options.y
     *            绘制的圆形的Y轴圆心位置（单位mm）。
     * @param {number} options.radius
     *            绘制的填充圆的半径（单位mm）。
     * @param {number} options.width
     *            绘制的填充圆所在区域的宽度。
     * @param {number} options.height
     *            绘制的填充圆所在区域的高度。
     * @return 打印成功与否？
     */
    api.fillCircle = function (options) {
        options = execUtils.processExecData(["x", "y", "radius"], arguments);
        options.fill = true;
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawCircle", options);
        } else if (execUtils.isIOS()) {
            return execUtils.execIOS("drawCircle", [
                options.x || 0,
                options.y || 0,
                (options.radius || 0) * 2,
                (options.radius || 0) * 2,
                0.1,
                options.fill,
            ]);
        }
    };

    /**
     * 绘制直线。
     *
     * @param {Object} options
     *            直线绘制相关参数；
     * @param {number} options.x1
     *            线的起点的水平位置（单位mm）。
     * @param {number} options.y1
     *            线的起点的垂直位置（单位mm）。
     * @param {number} options.x2
     *            线的终点的水平位置（单位mm）。
     * @param {number} options.y2
     *            线的终点的垂直位置（单位mm）。
     * @param {number} options.lineWidth
     *            线宽（单位mm）。线宽是向线的下方延伸的。
     * @param {number[]} options.dashLens
     *            点划线线段数组。
     * @return 打印成功与否？
     */
    api.drawLine = function (options) {
        options = execUtils.processExecData(["x1", "y1", "x2", "y2", "lineWidth"], arguments);
        options.x1 = typeof options.x1 === "number" ? options.x1 : options.x || 0;
        options.y1 = typeof options.y1 === "number" ? options.y1 : options.y || 0;
        options.x2 =
            typeof options.x2 === "number" ? options.x2 : options.width ? options.x1 + options.width : options.x1;
        options.y2 =
            typeof options.y2 === "number" ? options.y2 : options.height ? options.y1 + options.height : options.y1;
        if (execUtils.isAndroid()) {
            return execUtils.execAndroid("drawLine", options);
        } else if (execUtils.isIOS()) {
            let width =
                typeof options.width === "number" && options.width > 0 ? options.width : options.x2 - options.x1;
            let height =
                typeof options.height === "number" && options.height > 0 ? options.height : options.y2 - options.y1;
            const lineWidth = typeof options.lineWidth && options.lineWidth > 0 ? options.lineWidth : 0.5;
            const orientation = typeof options.orientation === "number" ? options.orientation : -1;
            if (width > 0 && height > 0) {
                //
            } else if (width > 0) {
                options.y1 -= lineWidth * 0.5;
                height = lineWidth;
            } else if (height > 0) {
                options.x1 -= lineWidth * 0.5;
                width = lineWidth;
            }
            return execUtils.execIOS("drawLine", [
                options.x1 || 0,
                options.y1 || 0,
                width,
                height,
                options.dashLens || [],
                orientation,
            ]);
        }
    };

    /**
     * 打印点划线。
     *
     * @param {Object} options
     *            点划线绘制相关参数；
     * @param {number} options.x1
     *            线的起点的水平位置（单位mm）。
     * @param {number} options.y1
     *            线的起点的垂直位置（单位mm）。
     * @param {number} options.x2
     *            线的终点的水平位置（单位mm）。
     * @param {number} options.y2
     *            线的终点的垂直位置（单位mm）。
     * @param {number} options.dashLen
     *            点划线线段长度的数组（单位mm）。
     * @param {number} options.lineWidth
     *            线宽（单位mm）。线宽是向线的下方延伸的。
     * @return 打印成功与否。
     */
    api.drawDashLine = function (options) {
        options = execUtils.processExecData(["x1", "y1", "x2", "y2", "dashLens", "lineWidth"], arguments);
        return api.drawLine(options);
    };

    module.exports = api;

